c
c                program regrid
c
c  input is a grid with a constant increment in x and y,
c  (dx,dy are not necessarily equal)
c  output is a grid with a specified increment, and optionally
c  covering a specified area.
c
c  the interpolater is either a 1-dimensional cubic spline applied first
c  in the y direction, then in x with a maximum grid dimension 2500x2500
c
c  or a linear interpolater with constant gradients along cell edges.
c
c      Mike Webring
c    Geophysics branch, USGS
c
	common /regrd/ buf(50000),z(2500)
	common /spl15/ wrk(12500)
	common /igrid/ nc,   nr,  xo,  dx,  yo,  dy
	common /ogrid/ nc2, nr2, xo2, dx2, yo2, dy2
	common /error/ ibackt, ibstep
	dimension      ibuf(50000)
	character*56   id, id2, blank, ifile, jfile, kfile
	character      pgm*8, pgm2*8, ans*8
	equivalence    ( buf, ibuf )
	data           kfile/'regrid.tmp'/, inp/5/,
	1              dval/'ffff7fff'x/,   isw,jsw,ksw/12,13,14/
c  dval is the maximum VAX floating word approx 1e37 and is the no data
	blank  = ' '
	ibackt = 0
	ibstep = 0

      pgm2 = 'regrid'

	type *, ' file to be interpolated '
	read( inp, 37 ) ifile
37	format( a )
	open( unit=isw, file=ifile, status='old', form='unformatted',
	1     readonly )

	call rdhdr( isw, id,pgm, nc,nr,nz, xo,dx,yo,dy, iproj,cm,bl )

c  type header record
	if ( iproj .gt. 0 ) then
	  type 210, id,p, nc,nr, xo,yo,dx,dy, iproj,cm,bl
210	  format( /,1x,a56,'  pgm:',a8,/,
	1       ' ncol=',i5,' nrow=',i5,/,
	1       ' xo,yo=',1p2g15.5,'   dx,dy=',2g15.5, /,
	1       ' projection #', i3, /,
	1       ' central meridian ', g15.5,  '  base lat', g15.5 )
	  else
	  type 211, id,p, nc,nr, xo,yo,dx,dy
211	  format(/,1x,a56,'  pgm:',a8,/,' ncol=',i5,' nrow=',i5,/,
	1        ' xo,yo=',1p2g15.5,'   dx,dy=',2g15.5)
	endif

	type *
	type *, ' output file  '
	read( inp, 37 )jfile
	type *, ' title ( <car ret> to keep input title ) '
	read( inp, 37 ) id2
	if ( id2 .eq. blank ) id2 = id

c  new increment
220	type *, ' new dx, dy'
	read *,  dx2, dy2
	if ( dx*dx2 .lt. 0.0 .or. dy*dy2 .lt. 0.0 ) then
	  type *, ' dx,dx2 or dy,dy2 have differing signs'
	  go to 220
	endif
	if ( dx2 / dx .lt. 0.1  .or.  dy2 / dy .lt. 0.1 ) then
	  type *, ' new dx or dy is less than 1/10 of the original'
	  type *, ' is this correct ? [y]'
	  if ( noyes() .ne. 0 ) then
	    type *, 'ok, just checking...'
	    else
	    go to 220
	  endif
	endif

c  adjust if close to integer subdivisor
	call intdx( dx, dx2 )
	call intdx( dy, dy2 )

	type *, ' change location or areal coverage ? [n]'
	if ( noyes() .eq. 1 ) then
	  type *,' enter new  x origin, y origin, ncol, nrow : '
	  read *, xo2, yo2, nc2, nr2
	else
	  xo2 = xo
	  yo2 = yo
	  nc2 = 1 + int( (float( nc-1 ) * dx ) / dx2 )
	  nr2 = 1 + int( (float( nr-1 ) * dy ) / dy2 )
	  type *,' new grid will be '
	  type *, nc2,' columns by ',nr2,' rows'
	endif

c  get interpolater type
	if ( nc .gt. 1500 .or.  nr .gt. 1500 .or.
	1   nc2 .gt. 1500 .or. nr2 .gt. 1500 ) then
	  type *, ' input or output grid size > 1500 col or rows'
	  type *, ' suggest linear interpolation '
	endif
	type *
130	type *, ' do you want linear or spline interpolation [spline]'
	accept 37, ans
	nch    = leftj( ans )
	linear = -1
	if ( ans(1:1) .eq. ' '  ) linear = 0
	if ( ans(1:1) .eq. 's'  .or. ans(1:1) .eq. 'S' ) linear = 0		
	if ( ans(1:1) .eq. 'l'  .or. ans(1:1) .eq. 'L' ) linear = 1		
	if ( linear .eq. -1 ) then
	  type *, ' enter linear, spline or <cr>'
	  go to 130
	endif

	if ( linear .eq. 0 ) then
	  type *, ' spline interpolation selected'
	  pgm2 = 'spline2d'
	  else
	  type *, ' linear interpolation selected'
	  pgm2 = 'linear2d'
	endif

	if ( linear .eq. 0 ) then
	if ( nc2 .gt. 2500  .or.  nr2 .gt. 2500 .or.
	1    nc  .gt. 2500  .or.  nr  .gt. 2500 ) then
	  type *,' input col=',nc, '   input row=',nr
	  type *,' output col=',nc2,'  output row=',nr2
	  type *,' spline in&out x&y max dimension is 2500'
	  type *,' switching to linear'
	  pgm2   = 'linear2d'
	  linear = 1
	endif
	endif

c  open output file, write header

	length = 1 + nc2
	if ( length .lt. 26 ) length = 26

	open( unit=jsw, file=jfile, status='new', form='unformatted',
	1     recl=length, recordtype='fixed', access='sequential' )

	write(jsw) id2,pgm2, nc2,nr2,nz, xo2,dx2,yo2,dy2,
	1          iproj,cm,bl

c  interpolate

	if ( linear .eq. 0 ) then
	  open( unit=ksw, file=kfile, status='scratch',
	1       form='unformatted', recl=400, access='direct' )
	  call spln2d( isw, jsw, ksw, dval )
	  else
	  n2 =  1 + nc
	  n3 = n2 + nc
	  n4 = n3 + nc
	  call lin2d( isw, jsw, dval, buf(1), buf(n2), ibuf(n3),
	1             buf(n4) )
	endif

	if ( ibackt .ne. 0 ) then
	  maxb = nc * nr2 + nc2 * nr2
	  type *, ' splint corrected indexing errors'
	  type *, ' backtrack=', ibackt, '  stepping', ibstep
	  type *, ' maximum possible backtrack =', maxb
	endif

	stop
	end
c***********************************************************************
	subroutine rdhdr( iunit, id, pgm, nc,nr,nz, xo,dx,yo,dy,
	1                        iproj, cmerid, baslat )
	character id*56, pgm*8
	read( iunit, err=9 )  id, pgm, nc,nr,nz, xo,dx,yo,dy,
	1                     iproj, cmerid, baslat
	return
9	rewind iunit
	read( iunit, err=99 ) id, pgm, nc,nr,nz, xo,dx,yo,dy
	iproj  = 0
	cmerid = 999.
	baslat = 999.
	return
99	type *, ' cannot read grid header'
	stop
	end
c***********************************************************************
	subroutine intdx( dx, dx2 )
c  assume the user was shooting for integral subdivision
c  if dx is an integral multiple of dx2 within .001
	if ( abs( dx * dx2 ) .lt. 1e-30 ) return
	t1   = abs ( dx/dx2 )
	t2   = abs ( dx2/dx )
	t    = amax1 ( t1, t2 )
	tint = aint( t + .001 )
	even = abs(  t - tint )
	if ( even .le. 0.001 ) then
	  if ( dx2 .lt. dx ) then
	    dx2 = dx / tint	
	    else
	    dx2 = dx * tint
	  endif
	endif
	return
	end
c***********************************************************************
	subroutine spln2d( isw, jsw, ksw, dval )
	common /regrd/ buf(50000), z(2500)
	common /spl15/ wrk(12500)
	common /igrid/  nc,  nr,  xo,  dx,  yo,  dy
	common /ogrid/ nc2, nr2, xo2, dx2, yo2, dy2
	m   = 1 + ( nc  - 1 ) / 20
	n   = 1 + ( nr  - 1 ) / 20
	n2  = 1 + ( nr2 - 1 ) / 20
	n   = max( n, n2 )
	m20 = m * 20
	n20 = n * 20
	call rio( buf, m, n, 1, 1, m20, -1, isw, ksw )
c
	ifill=0
	ic=1
	do 100 ibuf=1,m
c  assemble buffer of 20 columns
	loc=ibuf
	call rio(buf,n,0,loc,m,n20,1,isw,ksw)
c  spline in y
	iof=0
	do ir=1,20
	if(ifill.eq.0) then
	  do i=1,nr
	    z(i)=buf(i+iof)
	  enddo
	  call splint(nr,yo,dy,z,nr2,yo2,dy2,buf(1+iof))
	  if(ic.eq.nc) ifill=1
	else
	  do i=1,n20
	    buf(i+iof)=dval
	  enddo
	endif
	ic=ic+1
	iof=iof+n20
	enddo
c  output to same addresses
	call rio(buf,n,0,loc,m,n20,0,isw,ksw)
100	continue
c
c  spline in x, output final grid rows
	ir=1
	loc=1
	do ibuf=1,n2
	  call rio(buf,m,0,loc,1,m20,1,isw,ksw)
	  iof=1
	  do j=1,20
	    call splint(nc,xo,dx,buf(iof),nc2,xo2,dx2,z)
	    call rowio(nc2,z,0,jsw,jsw,iend)
	    if(ir.eq.nr2) go to 999
	    iof=iof+m20
	    ir=ir+1
	  enddo
	  loc=loc+m
	enddo
c
999	continue
	close(isw)
	close(jsw)
	close(ksw)
	stop
	end
c***********************************************************************
	subroutine lin2d( isw, jsw, dval, b1, b2, iflag, r )
c  grid linear interpolation assuming constant edge gradients
c  on four sides of the input cell.
c  f(ix+dx,iy+dy) = f(ix,iy) + a*dx + b*dy + c*dx*dy.
c  general placement of input and output grids.
c
	common /igrid/ nc,  nr,  xo,  dx,  yo,  dy
	common /ogrid/ nc2, nr2, xo2, dx2, yo2, dy2
	dimension b1(nc), b2(nc), iflag(nc), r(nc2)
c
c  switch internal coordinates to right cartesian
	if ( dx  .lt. 0 )  xo = -xo
	if ( dy  .lt. 0 )  yo = -yo
	if ( dx2 .lt. 0 ) xo2 = -xo2
	if ( dy2 .lt. 0 ) yo2 = -yo2
	dx  = abs( dx  )
	dx2 = abs( dx2 )
	dy  = abs( dy  )
	dy2 = abs( dy2 )
c
c  initialize output row
	do i = 1, nc2
	  r(i) = dval
	enddo
c
c  register input and output grids
c
	rewind isw
	read( isw )
	irow  = 0
	nout  = 0
	ywork = yo2
c	
c  output dval rows until output catches up to input.
	do while ( ywork .lt. yo )
	  write( jsw ) ywork, r
	  nout  = nout + 1
	  yout  = yo2  + dy2 * float( nout - 1 )
	  ywork = yout + dy2
	  if ( nout .eq. nr2 ) then
	    type *, ' output blank grid: yend, ynext=', yout, ywork
	    return
	  endif
	enddo
c
c  read input until yl & yu bracket ywork
	yu = -1.e35
	do while ( ywork .gt. yu )
	  read( isw, end = 777 ) xx, b1
	  irow = irow + 1
	  yl   = yo + dy * float( irow - 1 )
	  yu   = yl + dy
	enddo
	read( isw ) xx, b2
	irow = irow + 1
	yu   = yo + dy * float( irow - 1 )
c
	test = ( yu - yl ) / dy - 1.0
	if ( abs( test ) .gt. 0.1 ) go to 800
	if ( ywork .gt. yu   .or.   ywork .lt. yl ) go to 820
	call setflg( nc, b1, b2, iflag )
c
c  interpolation loop
c
	istop = 0
100	ywork = yo2 + dy2 * float( nout )
c
	if ( ywork .gt. yu ) then
c  update buffer
110	  do i = 1, nc
	    b1(i) = b2(i)
	  enddo
	  read( isw, end=120, iostat=ios ) xx, b2
120	  irow = irow + 1
	  yu   = yo + dy * float( irow - 1 )
	  yl   = yu - dy
	  if ( ios .eq. 0 ) then
	    if ( ywork .gt. yu ) go to 110
	    call setflg( nc, b1, b2, iflag )
	    else
	    istop = 1
	  endif
	endif
c
	if ( istop .eq. 0 ) then
c  interpolate row
	  call lin1d( dval, yl, yu, ywork, b1, b2, iflag, r )
	  else
c  output blank rows to end of output grid
	  do i = 1, nc2
	    r(i) = dval
	  enddo
	  do while ( 1 .eq. 1 )
	    write (jsw ) ywork, r
	    nout  = nout + 1
	    ywork = yo2 + dy2 * float( nout )
	    if ( nout .eq. nr2 ) return
	  enddo
	endif
c
c  output row
	write (jsw ) ywork, r
	nout = nout + 1
	if ( nout .lt. nr2 ) go to 100
	if ( nout .gt. nr2 ) go to 890
	return
c
777	type *, ' input eof - check grid origins'
	stop
800	type *, 'lin2d: buffer init,  test =', test
820	type *, 'lin2d: irow, yl, yu' , irow, yl, yu
890	type *, ' nout, ywork', nout, ywork
	stop
	end
c***********************************************************************
	subroutine setflg( nc, b1, b2, iflag )
	dimension b1(nc), b2(nc), iflag(nc)
	do i = 1, nc
	  iflag(i) = 0
	enddo
c
	if ( b1(1) .gt. 1.e29  .or.  b2(1) .gt. 1.e29 ) iflag(1) = 1
	do i = 2, nc
	  if ( b1(i) .gt. 1.e29  .or.  b2(i) .gt. 1.e29 ) then
	    iflag(i)   = 1
	    iflag(i-1) = 1
	  endif
	enddo
	return
	end
c***********************************************************************
	subroutine lin1d( dval, yl, yu, ywork, b1, b2, iflag, r )
c  linear interpolate profile
	common /igrid/ nc,  nr,  xo,  dx,  yo,  dy
	common /ogrid/ nc2, nr2, xo2, dx2, yo2, dy2
	dimension      b1(nc), b2(nc), iflag(nc), r(nc2)
	do i = 1, nc2
	  r(i) = dval
	enddo
c
	rdx  = 1.0 / dx
	dely = ( ywork - yl ) / dy
	ix   = int( ( xo2 - xo ) / dx ) - 1
	if ( ix .le. 0  ) ix = 1
	if ( ix .ge. nc ) go to 999
	xu   = xo + dx  * float( ix )
	xl   = xu - dx
c
	do 100 i2 = 1, nc2
	  x2 = xo2 + dx2 * float( i2 - 1 )
c
	  if ( x2 .lt. xl ) go to 100
10	  if ( x2 .gt. xu ) then
11	    ix = ix + 1
	    xu = xo + dx * float( ix )
	    xl = xu - dx
	    if ( ix .ge. nc ) go to 999
	    if ( x2 .gt. xu ) go to 11
	  endif
c
	if ( iflag(ix) .eq. 0  ) then
	  delx  = ( x2 - xl ) * rdx
	  ixu   =   ix + 1
	  a     =   b1(ixu) - b1(ix)
	  b     =   b2(ix)  - b1(ix)
	  c     =   b2(ixu) - b2(ix) - a
	  r(i2) =   b1(ix)  +  delx * a + dely * ( b + delx * c )
	endif

100	continue
c
999	return
	end
c***********************************************************************
	subroutine rio(z,nblk,nbuf,loc2,idloc,lenz,iop,isw,ksw)
c iop=-1 read grid, pad with dval's, and output with implied iop=1
c    =0 output 20x20 tranposed blocks
c    =1 input  20x20 block
c buffer z is dimensioned lenz x 20
c random access is nblk x nbuf, where 'loc' increments by 'idl'
	dimension hdr(16),w(400),z(1)
	logical more
	data dval/'ffff7fff'x/
	loc=loc2
	assign 999 to iret
c
	if(iop)1,2,3
1	assign 99 to iret
	more=.true.
	itot=0
	rewind(isw)
	read(isw) hdr,nc,nr,nz,xo,dx,yo,dy
	do 100 ibuf=1,nbuf
	do i=1,lenz*20
	  z(i)=dval
	enddo
	ireadz=1
	do irow=1,20
	  if(more) then
	    call rowio(nc,z(ireadz),-1,isw,isw,iend)
	    ireadz=ireadz+lenz
	    if(iend.eq.0) itot=itot+1
	    if(iend.ne.0 .or. itot.ge.nr) more=.false.
	  endif
	enddo
	  go to 2
99	  continue
100	continue
	if(itot.ne.nr) type *,' found only ',itot,' of ',nr,' rows'
	return
c
c  extract 20x20 block, tranpose, and output
2	do iblk=1,nblk
	iws=1
	izs=(iblk-1)*20+1
	do j=1,20
	  iw=iws
	  iz=izs
	  do i=1,20
	    w(iw)=z(iz)
	    iw=iw+20
	    iz=iz+1
	  enddo
	  iws=iws+1
	  izs=izs+lenz
	enddo
	write(ksw'loc) w
	loc=loc+idloc
	enddo
	go to iret,(99,999)
c
c  insert 20x20 block into buffer
3	do iblk=1,nblk
	read(ksw'loc) w
	iw=1
	izs=(iblk-1)*20+1
	do j=1,20
	  iz=izs
	  do i=1,20
	    z(iz)=w(iw)
	    iw=iw+1
	    iz=iz+1
	  enddo
	izs=izs+lenz
	enddo
	loc=loc+idloc
	enddo
c
999	return
	end
c***********************************************************************
	subroutine splint(nz,txo,tdx,z, nz2,txo2,tdx2,zint)
c  spline interpolation on defined intervals, no extrapolation
	common /spl15/ a(2500),b(2500),c(2500),
	1              w1(2500),w2(2500)
	common /error/ ibackt, ibstep
	dimension      d(2),z(nz),zint(nz2)
	data tol/.001/, d/0.,0./, dval,test/'ffff7fff'x,1.0e29/
c
	if ( tdx .eq. 0.0   .or.   tdx2 .eq. 0.0 ) go to 999
	do i = 1, nz2
	  zint(i) = dval
	enddo
	xo    = txo  * sign( 1.0, tdx  )
	dx    = abs( tdx )
	xo2   = txo2 * sign( 1.0, tdx2 )
	dx2   = abs( tdx2 )
	dxtol = amin1( dx*tol, dx2*tol )
	x1c   = xo  - dx
	x2c   = xo2 - dx2
	xend2 = float( nz2 ) * dx2 + x2c
c
	ie = 0
	xs = -test
	xe = -test

c  cycle the output array
	do 100 ioutp = 1, nz2
	xint = float(ioutp)*dx2 + x2c
	if ( xint .lt .xs ) go to 100
	if ( xint .ge. xe ) then
c  get start and ending indices of next input segment
5	next = ie + 1
	if ( next .ge. nz ) go to 999
	do is = next, nz
	  if ( z(is) .lt. test ) then
	    do ie = is + 1, nz
	      if ( z(ie) .gt. test ) go to 10
	    enddo
	    ie = nz + 1
	    go to 10
	  endif
	enddo
	go to 999
10	ie = ie - 1
	ni = ie - is + 1
	xs = float(is)*dx + x1c - dxtol
	if ( xs .gt. xend2 ) go to 999
	xe = float(ie)*dx + x1c + dxtol
	if ( xe .lt. xint ) go to 5
	if ( ni .lt. 2 ) go to 100
	  idata=is
	  x1 = xs      - dxtol
	  x2 = xs + dx + dxtol
	  if ( ni .gt. 2 ) then
	    call spln1(ni,dx,xdum,z(is),a,b,c,0,d,w1,w2)
	  else
	    a(1) = ( z(ie) - z(is) ) / dx
	    b(1) = 0.0
	    c(1) = 0.0
	  endif
	endif
c
c  find input interval that contains xint
	if ( xint.ge.x1  .and.  xint.lt.x2 ) go to 50
	idata1 = idata + 1
	if ( xint .ge. x2 ) then
	  if ( idata .eq. ie ) go to 100
	  do idata = idata1, ie - 1
	    x1 = float( idata ) * dx + x1c
	    x2 = x1 + dx + dxtol
	    x1 = x1      - dxtol
	    if ( xint .ge. x1  .and.  xint .lt. x2 ) go to 50
	  enddo
	else
	  if ( idata .eq. is ) go to 100
c  set back track error counter
	  ibackt = ibackt + 1
	  idata1 = idata - 1
	  do idata = idata1, is, -1
	    ibstep = ibstep + 1
	    x1 = float( idata ) * dx + x1c
	    x2 = x1 + dx + dxtol
	    x1 = x1      - dxtol
	    if ( xint .ge. x1   .and.   xint .lt. x2 ) go to 50
	  enddo
	endif
	go to 100
c
c  cubic interpolation
50	xd  =    xint - x1
	j   =  ( idata - is ) + 1
	zint(ioutp) = z(idata) + ( ( c(j)*xd + b(j) )*xd + a(j) )*xd
c
100	continue
999	return
	end
c***********************************************************************
	subroutine spln1(m,h,x,y,a,b,c,t,d,p,s)
c	coded by W. L. Anderson, U.S.G.S.
	dimension x(m),y(m),a(m),b(m),c(m),d(2),p(m),s(m)
	real mul
	integer t
	if(t.lt.0 .or. t.gt.1 .or. h.lt.0. .or. m.lt.3) go to 999
	n=m-1
	if(t.eq.0) go to 20
c  1ST DERIV BOUNDS GIVEN
	ne=n-1
	if(h)999,11,1
c EQUAL SPACING H>0  T=1
1	hh=3./h
	do 2 i=1,ne
	b(i)=4.
	c(i)=1.
	a(i)=1.
2	p(i)=hh*(y(i+2)-y(i))
	p(1)=p(1)-d(1)
	p(ne)=p(ne)-d(2)
c SOLN OF TRIDIAGONAL MATRIX ORDER NE
3	c(1)=c(1)/b(1)
	p(1)=p(1)/b(1)
	do 4 i=2,ne
	mul=1./(b(i)-a(i)*c(i-1))
	c(i)=mul*c(i)
4	p(i)=mul*(p(i)-a(i)*p(i-1))
c OBTAIN SPLINE COEF
	a(ne+t)=p(ne)
	i=ne-1
5	a(i+t)=p(i)-c(i)*a(i+t+1)
	i=i-1
	if(i.ge.1) go to 5
	if(t.eq.0) go to 6
	a(1)=d(1)
	a(m)=d(2)
6	if(h.eq.0.) go to 14
	hh=1./h
	do 7 i=1,n
	i1=i+1
	mul=hh*(y(i1)-y(i))
	b(i)=hh*(3.*mul-(a(i1)+2.*a(i)))
7	c(i)=hh*hh*(-2.*mul+a(i1)+a(i))
	return
c UNEQUAL SPACING H=0  T=1
11	do 12 i=1,n
	i1=i+1
12	s(i1)=x(i1)-x(i)
	do 13 i=1,ne
	i1=i+1
	i2=i+2
	b(i)=2.*(s(i1)+s(i2))
	c(i)=s(i1)
	a(i)=s(i2)
	sq1=s(i1)*s(i1)
	sq2=s(i2)*s(i2)
13	p(i)=3.*(sq1*(y(i2)-y(i1))+sq2*(y(i1)-y(i)))/(s(i1)*s(i2))
c       B(I)=2.*(S(I+1)+S(I+2))
c       C(I)=S(I+1)
c       A(I)=S(I+2)
c13     P(I)=3.*(S(I+1)**2*(Y(I+2)-Y(I+1))+S(I+2)**2*(Y(I+1)-Y(I)))/
c       1  (S(I+1)*S(I+2))
	p(1)=p(1)-s(3)*d(1)
	p(ne)=p(ne)-s(n)*d(2)
	go to 3
14	do 15 i=1,n
	i1=i+1
	hh=1./s(i1)
	hs=hh*hh
	mul=(y(i1)-y(i))*hs
	b(i)=3.*mul-(a(i1)+2.*a(i))*hh
15	c(i)=-2.*mul*hh+(a(i1)+a(i))*hs
	return
c  2ND DERIV BOUNDS GIVEN
20	ne=n+1
	if(h)999,31,21
c  EQUAL SPACING H>0  T=0
21	hh=3./h
	do 22 i=2,n
	b(i)=4.
	c(i)=1.
	a(i)=1.
22	p(i)=hh*(y(i+1)-y(i-1))
	b(1)=2.
	b(ne)=2.
	c(1)=1.
	c(ne)=1.
	a(ne)=1.
	p(1)=hh*(y(2)-y(1))-.5*h*d(1)
	p(ne)=hh*(y(m)-y(n))+.5*h*d(2)
	go to 3
c  UNEQUAL SPACING H=0  T=0
31	do 32 i=1,n
	i1=i+1
32	s(i1)=x(i1)-x(i)
	n1=n-1
	do 33 i=1,n1
	i1=i+1
	i2=i+2
	b(i1)=2.*(s(i1)+s(i2))
	c(i1)=s(i1)
	a(i1)=s(i2)
	sq1=s(i1)*s(i1)
	sq2=s(i2)*s(i2)
33	p(i1)=3.*(sq1*(y(i2)-y(i1))+sq2*(y(i1)-y(i)))/(s(i1)*s(i2))
c       B(I+1)=2.*(S(I+1)+S(I+2))
c       C(I+1)=S(I+1)
c       A(I+1)=S(I+2)
c33     P(I+1)=3.*(S(I+1)**2*(Y(I+2)-Y(I+1))+S(I+2)**2*(Y(I+1)-Y(I)))/
c       1 (S(I+1)*S(I+2))
	b(1)=2.
	b(ne)=2.
	c(1)=1.
	c(ne)=1.
	a(ne)=1.
	p(1)=3.*(y(2)-y(1))/s(2)-.5*s(2)*d(1)
	p(ne)=3.*(y(m)-y(n))/s(m)+.5*s(m)*d(2)
	go to 3
999	m=-abs(m)
	return
	end
c***********************************************************************
	subroutine rowio(n,z,iop,idev,jdev,iend)
c  read iop<0, write iop=0, r&w iop=1
	dimension z(n)
	iend=0
	if(iop)1,2,1
1	read(idev,end=10) y,z
	if(iop)9,9,2
2	write(jdev) y,z
9	return
10	iend=1
	return
	end
c***********************************************************************
	function noyes()
c  returns -1 for indeterminate, 0 for no, 1 for yes
	character  inp*4, blank*4
	data       blank/ '    ' /

	icnt  = 0
	noyes = -1

1	inp   = ' '
	read ( 5, 2, end=5, err=5 ) inp
2	format( a )
	if ( inp .eq. blank ) return

	do 3 n = 1, 4
	  if ( inp(n:n) .ne. ' ' ) then
	    if ( inp(n:n).eq.'y' .or. inp(n:n).eq.'Y' ) noyes = 1
	    if ( inp(n:n).eq.'n' .or. inp(n:n).eq.'N' ) noyes = 0
	    if ( noyes .gt. -1 ) return
	    go to 5
	  endif
3	continue

5	if ( icnt .lt. 3 ) then
	  icnt = icnt + 1
	  write( 6, * ) ' <cr>, no, or yes :'
	  go to 1
	endif

	write( 6, * ) ' count exceeded, returning indeterminate state '
9	return
	end
c***********************************************************************
	function leftj(a)
c  left justifies a string and returns the position
c  of the last nonblank character
	character a*(*)
	n = len(a)
	ich = ichar(a(1:1))
	if (ich.ne.0 .and. ich.ne.32) go to 15
	do 1 m = 2, n
	ich = ichar(a(m:m))
1	if (ich.ne.0 .and. ich.ne.32) go to 5
	leftj = 0
	return
c
5	i2 = 0
	do 10 i = m, n
	i2 = i2 + 1
	a(i2:i2) = a(i:i)
10	continue
	do 11 i3 = i2+1, n
11	a(i3:i3) = ' '
	n = n-m+1
c
15	do 20 leftj = n, 1, -1
	ich = ichar(a(leftj:leftj))
20	if (ich.ne.0 .and. ich.ne.32) go to 25
25	return
	end
