C *******************************************************************
C  TAYLOR.FOR
C
C  DRIVER FOR LEVEL-TO-DRAPE AND DRAPE-TO-LEVEL CONTINUATION USING
C  TAYLOR SERIES EXPANSION.
C
C  V.J.S. GRAUCH, USGS, MARCH 1984.  Refer to Open-File Report 84-501
C  for detailed instructions and theory.
C
C     F     =     INPUT DATA
C     G     =     1ST VERTICAL DERIVATIVE OF F
C     H     =     2ND VERTICAL DERIVATIVE OF F
C     Z     =     SURFACE DATA
C     IOPT  =     1 IF LEVEL-TO-DRAPE CHOSEN, 2 IF DRAPE-TO-LEVEL
C
C *******************************************************************
      CHARACTER*50 IFILE,OFILE,OUTFILE,IFILEA,IFILEB,SFILE
      CHARACTER*56 TITLE
      DIMENSION WORK(2048),F(2,1024,16),G(2,1024,16),H(2,1024,16),
     1 Z(1024),ID(14),PGM(2)
      COMMON/L2DPARMS/FLEVEL,CONVF,ZLEVEL,IFILT,W1,W2,IDVAL,NADD,NTERMS,
     1IFILE,OFILE,SFILE,TITLE,ITER
      COMMON/L2DINFO/P1,P2,NZ,YO,XO,DX,DY,NCOL,NROW
      COMMON/UNITS/IW,IR,IN,IOUT,ICMD,IS,IUNITA,IUNITB,IUNITC,IF,IUNITG,
     1IUNITH
      DATA IW/6/,IR/5/,IN/10/,IOUT/12/,ICMD/9/,IS/11/,IUNITA/20/,
     1IUNITB/21/,IUNITC/22/,IF/14/,IUNITG/23/,IUNITH/24/,LNRI/16/
1     WRITE(IW,800)
800   FORMAT(1X,'1 - LEVEL TO DRAPE CONTINUATION'/' 2 - DRAPE TO LEVEL
     1 CONTINUATION'/8X,'ENTER OPTION: ',$)
      READ(IR,*) IOPT
      IF(IOPT.LT.1.OR.IOPT.GT.2) GO TO 1
C
C  SET UP INPUT PARAMETERS
C
      CALL SETUP(IOPT)
C  FIND NO. OF ROWS FOR FFT AND EXTEND COLS BY NADD
      CALL EXTEND(NADD,N1,N2,ID2,LNRI,NRI,NXA)
      IF(IOPT.EQ.2.OR.IOPT.EQ.4) GO TO 20
C
C   LEVEL TO DRAPE OPERATION
C
      P1='*L T'
      P2='O D*'
      OPEN(IOUT,FILE=OFILE,FORM='UNFORMATTED',STATUS='NEW')
      WRITE(IOUT) TITLE,P1,P2,NCOL,NROW,NZ,XO,DX,YO,DY
      WRITE(IW,804)
804   FORMAT(/' COMPUTING...'/)
      CALL L2D(F,G,H,WORK,ID2,NRI,N1,N2,Z,NXA)
      CLOSE(IN)
      CLOSE(IS)
      CLOSE(IOUT)
      GO TO 300
C
C  DRAPE TO LEVEL OPERATION
C
20    FINLEVEL=FLEVEL
      FLEVEL=ZLEVEL
      ISAVE=0
C GET 1ST APPROX BY USING LEVEL-TO-DRAPE WITH NEG. Z
      CONVF=-CONVF
      IF(ITER.EQ.-1) THEN
         ISAVE=1
         ITER=2
      ENDIF
      OPEN(IOUT,FILE='F1.TMP',STATUS='NEW',FORM='UNFORMATTED')
      WRITE(IOUT) ID,PGM,NCOL,NROW,NZ,XO,DX,YO,DY
      WRITE(IW,805)
805   FORMAT(/28X,'RECOVERY ERROR MAGNITUDE'/' ITERATION',10X,'MAX',
     1 15X,'MEAN',12X,'STD DEV.'/)
      CALL L2D(F,G,H,WORK,ID2,NRI,N1,N2,Z,NXA)
      CLOSE(IN)
      REWIND IS
      CLOSE(IOUT)
C
C  GET NEXT APPROXIMATIONS
C
      CONVF=-CONVF
      OUTFILE='F1D.TMP'
      IFILEA='F1.TMP'
      K=0
25    K=K+1
      OPEN(IN,FILE=IFILEA,STATUS='OLD',FORM='UNFORMATTED',READONLY)
      READ(IN) ID,PGM,NCOL,NROW,NZ,XO,DX,YO,DY
      READ(IS) ID,PGM,NCOL,NROW,NZ,XO,DX,YO,DY
      OPEN(IOUT,FILE=OUTFILE,STATUS='NEW',FORM='UNFORMATTED')
      WRITE(IOUT) ID,PGM,NCOL,NROW,NZ,XO,DX,YO,DY
      CALL L2D(F,G,H,WORK,ID2,NRI,N1,N2,Z,NXA)
      CLOSE(IN)
      CLOSE(IOUT)
C  FIND ERROR AND ADD ERROR TO ONE APPROX TO GET NEXT APPROX
      IF(K.EQ.2) GO TO 32
      OUTFILE='F2.TMP'
      IFILEA='F1D.TMP'
      IFILEB='F1.TMP'
      GO TO 33
32    IFILEA='F2D.TMP'
      IFILEB='F2.TMP'
      OUTFILE='F3.TMP'
33    IF(K.NE.ITER) GO TO 35
      CLOSE(IS)
      IF(ABS(FINLEVEL-ZLEVEL).GT.0.0001) GO TO 36
      OPEN(IOUT,FILE=OFILE,FORM='UNFORMATTED',STATUS='NEW')
      P1='*D T'
      P2='O L*'
      WRITE(IOUT) TITLE,P1,P2,NCOL,NROW,NZ,XO,DX,YO,DY
      GO TO 40
35    REWIND(IS)
36    OPEN(IOUT,FILE=OUTFILE,FORM='UNFORMATTED',STATUS='NEW')
      WRITE(IOUT) ID,PGM,NCOL,NROW,NZ,XO,DX,YO,DY
40    OPEN(IN,FILE=IFILE,STATUS='OLD',FORM='UNFORMATTED',READONLY)
      OPEN(IUNITA,FILE=IFILEA,STATUS='OLD',FORM='UNFORMATTED')
      OPEN(IUNITB,FILE=IFILEB,STATUS='OLD',FORM='UNFORMATTED')
      READ(IN) ID,PGM,NCOL,NROW,NZ,XO,DX,YO,DY
      READ(IUNITA) ID,PGM,NCOL,NROW,NZ,XO,DX,YO,DY
      READ(IUNITB) ID,PGM,NCOL,NROW,NZ,XO,DX,YO,DY
      ERRMAX=-1.E+38
      SUMERR=0.0
      SUMERR2=0.0
      EN=0.0
      DO 50 J=1,NROW
      READ(IN) DUM,(F(1,I,1),I=1,NCOL)
      READ(IUNITA) DUM,(G(1,I,1),I=1,NCOL)
      READ(IUNITB) DUM,(H(1,I,1),I=1,NCOL)
      DO 45 I=1,NCOL
      IF(F(1,I,1).GT.1.E+38) GO TO 45
      ERR=F(1,I,1)-G(1,I,1)
      IF(ABS(ERR).GT.ERRMAX) ERRMAX=ABS(ERR)
      SUMERR=SUMERR+ABS(ERR)
      IF(SUMERR.GE.1.E+20) GO TO 299
      SUMERR2=SUMERR2+ERR*ERR
      EN=EN+1.0
      F(1,I,1)=ERR+H(1,I,1)
45    CONTINUE
      WRITE(IOUT) DUM,(F(1,I,1),I=1,NCOL)
50    CONTINUE
      SUMERR=SUMERR/EN
      STDDEV=(SUMERR2-(SUMERR*SUMERR)/EN)/(EN-1.0)
      STDDEV=SQRT(STDDEV)
      WRITE(IW,802) K,ERRMAX,SUMERR,STDDEV
802   FORMAT(5X,I1,9X,G14.6,2(5X,G14.6))
      CLOSE(IN)
      CLOSE(IUNITA,STATUS='DELETE')
      IF(ISAVE.NE.0) CLOSE(IUNITB)
      IF(ISAVE.EQ.0) CLOSE(IUNITB,STATUS='DELETE')
      CLOSE(IOUT)
      IF(K.EQ.ITER) GO TO 100
      IFILEA=OUTFILE
      OUTFILE='F2D.TMP'
      GO TO 25
100   IF(ISAVE.NE.0) WRITE(IW,803)
803   FORMAT(/' 1ST-ITER. APPROX. (ON Z REF. LEVEL) IS IN FILE F1.TMP')
      IF(ABS(FINLEVEL-ZLEVEL).LE.0.0001) GO TO 300
C  LEVEL-TO-LEVEL CONTINUATION IF NEEDED FOR FINAL LEVEL FILE
      FLEVEL=ZLEVEL
      ZLEVEL=FINLEVEL
      NTERMS=1
      OPEN(IN,FILE=OUTFILE,STATUS='OLD',FORM='UNFORMATTED')
      READ(IN) ID,PGM,NCOL,NROW,NZ,XO,DX,YO,DY
      OPEN(IOUT,FILE=OFILE,STATUS='NEW',FORM='UNFORMATTED')
      P1='*D T'
      P2='O L*'
      WRITE(IOUT) TITLE,P1,P2,NCOL,NROW,NZ,XO,DX,YO,DY
      CALL L2D(F,G,H,WORK,ID2,NRI,N1,N2,Z,NXA)
      CLOSE(IN,STATUS='DELETE')
      CLOSE(IOUT)
      GO TO 300
299   WRITE(IW,806)
806   FORMAT(' METHOD IS RAPIDLY DIVERGING.  ABORT.')
      CLOSE(IUNITA,STATUS='DELETE')
      CLOSE(IUNITB,STATUS='DELETE')
      CLOSE(IOUT,STATUS='DELETE')
300   STOP
      END
C **********************************************************************
C
      SUBROUTINE SETUP(IOPT)
C SETS UP THE INPUT PARAMETERS FOR THE LEVEL TO DRAPE OR
C DRAPE TO LEVEL OPERATIONS.  USED WITH TAYLOR.FOR
C
C ********************************************************************
      CHARACTER ANSWER*1,TITLE*56
      CHARACTER*50 IFILE,OFILE,SFILE,COMMAND
      COMMON/L2DPARMS/FLEVEL,CONVF,ZLEVEL,IFILT,W1,W2,
     1IDVAL,NADD,NTERMS,IFILE,OFILE,SFILE,TITLE,ITER
      COMMON/L2DINFO/P1,P2,NZ,YO,XO,DX,DY,NCOL,NROW
      COMMON/UNITS/IW,IR,IN,IOUT,ICMD,IS,IUNITA,IUNITB,IUNITC,IF,IUNITG,
     1IUNITH
      DIMENSION ID(14),PGM(2),Z(1024)
C
C
      IFILE=' '
      OFILE=' '
      SFILE=' '
      TITLE='THIS IS THE DEFAULT TITLE'
1     WRITE(IW,800)
800   FORMAT(' COMMAND OR COMMAND FILE NAME: ',$)
      READ(IR,801) COMMAND
801   FORMAT(A50)
      IF(COMMAND.EQ.'EXIT'.OR.COMMAND.EQ.'EXIT') GO TO 300
      IF(COMMAND.EQ.'HELP'.OR.COMMAND.EQ.'HELP'.OR.COMMAND.EQ.' ') THEN
         WRITE(IW,888)
         GO TO 1
  888 FORMAT(' COMMANDS ARE'/,5X,'TERM - ENTER THE PARAMETERS DESIRED
     1INTERACTIVELY AND CREATE .CMD FILE AT'/,10X,' THE SAME TIME.',/,
     2 5X,'HELP'/ 5X,'EXIT'/' ANYTHING ELSE ENTERED WILL BE TAKEN TO
     3 BE A COMMAND FILE NAME.')
      ENDIF
      IF(COMMAND.NE.'TERM'.AND.COMMAND.NE.'TERM') GO TO 50
C
C  INTERACTIVELY SET UP PARAMETERS AND COMMAND FILE.  COMMAND FILE WILL
C  BE NAMED TAYLOR.CMD
      COMMAND='TAYLOR.CMD'
      WRITE(IW,802)
802   FORMAT(/,' BEGIN ENTERING PARMS INTERACTIVELY.',/,' WANT ME TO
     1 ASK VERBOSE (AS OPPOSED TO BRIEF) QUESTIONS? '$)
      READ(IR,803) ANSWER
803   FORMAT(A1)
      IF(ANSWER.EQ.'Y'.OR.ANSWER.EQ.'Y') IOPT=IOPT+2
C ASK FOR PARMS
      IF(IOPT.EQ.2.OR.IOPT.EQ.4) WRITE(IW,855)
855   FORMAT(/' WILL EITHER INPUT FILE HAVE DVALS? '$)
      IF(IOPT.EQ.1.OR.IOPT.EQ.3) WRITE(IW,856)
856   FORMAT(/' WILL INPUT DATA FILE HAVE DVALS? ',$)
      READ(IR,803) ANSWER
      IDVAL=0
      IF(ANSWER.EQ.'Y'.OR.ANSWER.EQ.'Y') IDVAL=1
      IF(IOPT.GT.2) WRITE(IW,709)
709   FORMAT(/'  WHAT IS THE FACTOR TO CONVERT THE UNITS OF MEASURE OF
     1 THE SURFACE INTO THE',/,'  UNITS OF MEASURE OF THE GRID INTERVAL
     2 (E.G. .001 IF SURFACE IS IN M AND'/'  GRID UNITS IN KM).  ENTER
     3 THE NEGATIVE OF THIS NUMBER IF THE SENSE OF THE'/'  SURFACE FILE
     4 IS Z POSITIVE DOWN.')
      WRITE(IW,809)
809   FORMAT(' CONVERSION FACTOR = ',$)
      READ(IR,*) CONVF
      IF(IOPT.EQ.2.OR.IOPT.EQ.4) GO TO 10
      IF(IOPT.EQ.3) WRITE(IW,706)
706   FORMAT(/'  GIVE THE LEVEL AT WHICH THE INPUT DATA WERE OBSERVED IN
     1 THE SAME UNITS AS THOSE'/'  OF THE DRAPING SURFACE.  (REMEMBER TO
     2 BE CONSISTENT WITH WHETHER UP OR DOWN'/'  IS THE POSITIVE
     3 DIRECTION.)')
      WRITE(IW,806)
806   FORMAT(' FLEVEL= ',$)
      READ(IR,*) FLEVEL
10    IF(IOPT.GT.2) WRITE(IW,711)
711   FORMAT(/'  THE TAYLOR SERIES EXPANDS ABOUT A Z REFERENCE LEVEL
     1 THAT SHOULD BEST BE NEAR'/'  THE MEAN OF THE DRAPING SURFACE SO
     2 THAT THE DEVIATION OF THE SURFACE'/'  ABOVE AND BELOW THAT LEVEL
     3 IS THE LEAST AMOUNT.  ENTER THE LEVEL IN THE'/'  SAME UNITS USED
     4 FOR THE DRAPING SURFACE.')
      WRITE(IW,811)
811   FORMAT(' ENTER ZLEVEL (1.E+38 TO USE DEFAULT):  ',$)
      READ(IR,*) ZLEVEL
      KFLAG=0
      IF(ZLEVEL.GE.1.E+38) KFLAG=1
15      IF(IOPT.EQ.1.OR.IOPT.EQ.3) GO TO 17
      IF(IOPT.GT.2) WRITE(IW,712)
712   FORMAT(/'  ENTER THE LEVEL TO WHICH TO CONTINUE THE DATA IN THE
     1 SAME UNITS'/'  AS THE DRAPING SURFACE UNITS.')
      WRITE(IW,812)
812   FORMAT(' FLEVEL = ',$)
      READ(IR,*) FLEVEL
17      NTERMS=3
      IF(IOPT.EQ.1.OR.IOPT.EQ.3) GO TO 18
      IF(IOPT.GT.2) WRITE(IW,730)
730   FORMAT(/'  READ DOCUMENTATION FOR EXAMPLES ON WHEN TO USE
     1 2 OR 3 TERMS OF THE TAYLOR'/'  SERIES EXPANSION.')
      WRITE(IW,830)
830   FORMAT(' WANT 2 OR 3 TERMS IN TAYLOR EXPANSION? ',$)
      READ(IR,*) NTERMS
18    IF(IOPT.GT.2) WRITE(IW,713)
713   FORMAT(/'  BANDPASS (LOWPASS) FILTERING IS OFTEN DESIRABLE DURING
     1 THE 1ST & 2ND VERT.'/'  DERIVATIVE-TAKING OF THE TAYLOR SERIES
     2 EXPANSION OR IN THE CONTINUATION'/'  OF FLEVEL TO ZLEVEL OR VICE
     3 VERSA.  OF COURSE IF ZLEVEL=FLEVEL, FILTERING'/'  THE CONTINU
     4ATION IS NOT DESIRABLE.')
20    WRITE(IW,813)
813   FORMAT(' ENTER FILTERING OPTION (-1 FOR HELP): ',$)
      READ(IR,*) IFILT
      IF(IFILT.EQ.0) GO TO 30
      IF(IFILT.GT.0.AND.IFILT.LE.3) GO TO 25
      WRITE(IW,833)
833   FORMAT('   0 - NO FILTERING'/'   1 - FILTER DERIVS ONLY'/'   2 -
     1 FILTER DERIVS AND CONTINUATION'/'   3 - FILTER CONTINUATION
     2 ONLY')
      GO TO 20
25    IF(IOPT.GT.2) WRITE(IW,714)
714   FORMAT(/'  TWO WAVELENGTHS W1<=W2 DESCRIBE A (TAPERED) LOWPASS
     1 FILTER.  REMEMBER',/,'  WAVELENGTH=2 * ANOMALY WIDTH.  DEFAULTS:
     2  W1=2*GRID INTERVAL, W2=W1+2*GRID'/'  INTERVAL.  W1 &
     3 W2 ARE IN SAME UNITS AS GRID INTERVAL.')
      WRITE(IW,814)
814   FORMAT(' ENTER W1 AND W2 (TYPE 0 0 TO USE DEFAULT) ',$)
      READ(IR,*) W1,W2
      IF(W1.GT.W2) GO TO 25
      IF(W2.NE.0.) GO TO 30
      W1=1.E+38
      W2=1.E+38
      KFLAG=KFLAG+2
30    IF(IOPT.GT.2) WRITE(IW,724)
724   FORMAT(/'  IT OFTEN HELPS THE FFT OPERATION TO EXTEND THE NUMBER
     2 OF ROWS AND',/,'  COLUMNS BY A SMALL PROPORTION OF THE TOTAL
     3 GRID.  ENTER THE NUMBER TO'/'  ADD TO THE COLS AND ROWS:')
      WRITE(IW,824)
824   FORMAT(' NADD = ',$)
      READ(IR,*) NADD
      IF(IOPT.EQ.1.OR.IOPT.EQ.3) GO TO 32
31    IF(IOPT.GT.2) WRITE(IW,725)
725   FORMAT(/'  THE APPROXIMATION OF DATA ON THE LEVEL IS FOUND THROUGH
     1 SUCCESSIVE'/'  ITERATIONS (1 OR 2).  MORE ITERATIONS DOESN''T
     2 NECCESSARILY MEAN A BETTER'/'  APPROXIMATION THOUGH (SEE DOCU
     3MENTATION).  ENTER ITER=-1 IF'/'  YOU WANT 2 ITERATIONS BUT ALSO
     4 WISH TO SAVE THE GRID OF THE 1ST ITERATION.')
      WRITE(IW,825)
825   FORMAT(' ENTER ITER: ',$)
      READ(IR,*) ITER
      IF(ITER.EQ.-1.OR.ITER.LE.2.AND.ITER.GE.1) GO TO 32
      WRITE(IW,725)
      GO TO 31
C
C  CREATE COMMAND FILE IF REQUESTED
C
32    IF(IOPT.GT.2) WRITE(IW,715)
715   FORMAT(/' NOW I CAN CREATE A COMMAND FILE WITH ALL THE
     1 PARAMETERS YOU PICKED',/,' SO THAT YOU CAN EDIT AND/OR USE IT
     2 FOR A LATER PROGRAM RUN.')
      WRITE(IW,860)
860   FORMAT(/' WANT TO SAVE THIS INFO IN COMMAND FILE? ',$)
      READ(IR,803) ANSWER
      IF(ANSWER.NE.'Y'.AND.ANSWER.NE.'Y') GO TO 51
      OPEN(ICMD,FILE=COMMAND,STATUS='UNKNOWN',FORM='FORMATTED',
     1CARRIAGECONTROL='LIST')
      IF(KFLAG.GT.0) GO TO 35
      WRITE(ICMD,818) FLEVEL,CONVF,NTERMS,ZLEVEL,IFILT,W1,W2,NADD,IDVAL
818   FORMAT(' &PARMS  FLEVEL=',G14.6,',CONVF=',G14.6,',NTERMS=',I1,
     2',ZLEVEL=',G14.6,',',/,' IFILT=',I1,',W1=',G14.6,',W2=',G14.6,/,
     3' NADD=',I2,',IDVAL=',I1,' &')
      GO TO 40
35    IF(KFLAG.GT.1) GO TO 37
      WRITE(ICMD,819) FLEVEL,CONVF,NTERMS,IFILT,W1,W2,NADD,IDVAL
819   FORMAT(' &PARMS FLEVEL=',G14.6,',CONVF=',G14.6,',NTERMS=',I1,
     1/,' IFILT=',I1,',W1=',G14.6,',W2=',G14.6,/,' NADD=',I2,',IDVAL=',
     2 I1,' &')
      GO TO 40
37    IF(KFLAG.EQ.2) GO TO 38
      WRITE(ICMD,719) FLEVEL,CONVF,NTERMS,IFILT,NADD,IDVAL
719   FORMAT(' &PARMS FLEVEL=',G14.6,',CONVF=',G14.6,',NTERMS=',I1,
     1/,' IFILT=',I1,',NADD=',I2,',IDVAL=',I1,' &')
      GO TO 40
38    WRITE(ICMD,720) FLEVEL,CONVF,NTERMS,ZLEVEL,IFILT,NADD,IDVAL
720   FORMAT(' &PARMS FLEVEL=',G14.6,',CONVF=',G14.6,',NTERMS=',I1,
     1/' ZLEVEL=',G14.6,',IFILT=',I1,',NADD=',I2,',IDVAL=',I1,' &')
40    CLOSE(ICMD)
      WRITE(IW,820)
820   FORMAT(/' **COMMAND FILE TAYLOR.CMD CREATED**')
      GO TO 51
C
C  GET PARMS FROM EXISTING COMMAND FILE
C
C  SET DEFAULT VALUES OF PARMS
50    NADD=0
      CONVF=1.
      IFILT=0
      FLEVEL=0.
      ZLEVEL=1.E+38
      W1=1.E+38
      W2=1.E+38
      NTERMS=3
      ITER=-3
      OPEN(ICMD,FILE=COMMAND,FORM='FORMATTED',STATUS='OLD',
     1READONLY,CARRIAGECONTROL='LIST',ERR=299)
      CALL GETPARMS(ICMD,IW)
C CHECK FOR PARAMETER ERRORS
      IF(IFILT.LT.0.OR.IFILT.GT.3) GO TO 290
      IF(IFILT.NE.0.AND.W1.GT.W2) GO TO 290
      IF(IDVAL.NE.0.AND.IDVAL.NE.1) GO TO 290
      IF(NADD.LT.0) GO TO 290
      IF(NTERMS.LT.1.OR.NTERMS.GT.3) GO TO 290
      IF(IOPT.EQ.1.OR.IOPT.EQ.3) GO TO 51
      IF(ITER.EQ.-1.OR.ITER.EQ.2.OR.ITER.EQ.1) GO TO 51
      WRITE(IW,825)
      READ(IR,*) ITER
49    IF(ITER.EQ.-1.OR.ITER.EQ.2.OR.ITER.EQ.1) GO TO 51
      WRITE(IW,725)
      WRITE(IW,825)
      READ(IR,*) ITER
      GO TO 49
C PROMPT FOR INPUT FILES
51    IF(IFILE.NE.' ') GO TO 55
52    IF(IOPT.EQ.1.OR.IOPT.EQ.3) WRITE(IW,704)
704   FORMAT(/' INPUT FILE (LEVEL SURVEY): ',$)
      IF(IOPT.EQ.2.OR.IOPT.EQ.4) WRITE(IW,804)
804   FORMAT(/' INPUT FILE (DRAPED SURVEY): ',$)
      READ(IR,801) IFILE
55    OPEN(IN,FILE=IFILE,STATUS='OLD',FORM='UNFORMATTED',READONLY,
     1ERR=297)
      READ(IN)ID,PGM,NCOL,NROW,NZ,XO,DX,YO,DY
      IF(DX.NE.DY) GO TO 295
      IF(SFILE.NE.' ') GO TO 57
56    WRITE(IW,807)
807   FORMAT(/' ENTER DRAPING OR IRREGULAR SURFACE GRID: ',$)
      READ(IR,801) SFILE
57    OPEN(IS,FILE=SFILE,FORM='UNFORMATTED',READONLY,STATUS='OLD',
     1ERR=298)
      READ(IS) ID,PGM,NCOL2,NROW2,NZ,XO2,DX2,YO2,DY2
      IF(ABS(XO-XO2).GT.0.001.OR.ABS(YO-YO2).GT.0.001) GO TO 293
      IF(ABS(DX-DX2).GT.0.001*ABS(DX).OR.DX2.NE.DY2) GO TO 293
      IF(NROW.NE.NROW2.OR.NCOL.NE.NCOL2) GO TO 293
      IF(OFILE.NE.' ') GO TO 59
      WRITE(IW,815)
815   FORMAT(/' OUTPUT FILE: ',$)
      READ(IR,801)OFILE
59    IF(TITLE.NE.'THIS IS THE DEFAULT TITLE') GO TO 100
      WRITE(IW,816)
816   FORMAT(' ENTER TITLE')
      READ(IR,817) TITLE
817   FORMAT(A56)
C CHECK ZERO LEVEL OR SET IT IF DEFAULT USED
100   CALL REFLVL(IS,RECLEVEL,NROW,NCOL,DX,IW,CONVF,Z)
      IF(ZLEVEL.EQ.1.E+38) GO TO 105
      IF(ABS((ZLEVEL-RECLEVEL)*CONVF).GT.ABS(1.5*DX)) WRITE(IW,821)
     1 ZLEVEL,RECLEVEL
821   FORMAT(/' **WARNING: ZERO LEVEL OF ',G14.6,' IS MORE THAN 1.5*DX
     1 AWAY FROM',/,'    RECOMMENDED LEVEL OF ',G14.6)
      GO TO 110
105   ZLEVEL=RECLEVEL
      WRITE(IW,822) ZLEVEL
822   FORMAT(/' USING Z REF. LEVEL OF ',G14.6)
C  SET W1 AND W2 IF DEFAULT VALUES PICKED
110   IF(W1.LT.1.E+38.AND.W2.LT.1.E+38.OR.IFILT.EQ.0) GO TO 300
      W1=2*DX
      W2=4*DX
      WRITE(IW,823) W1,W2
823   FORMAT(' USING W1,W2= ',G14.6,1X,G14.6)
      GO TO 300
C
C ERROR  MESSAGES
C
290   WRITE(IW,292)
292   FORMAT(' COMMAND FILE HAS PARAMETER ERROR.  ABORT.')
      STOP
293   WRITE(IW,294)
294   FORMAT(' GRIDS DON''T MATCH')
      STOP
295   WRITE(IW,296)
296   FORMAT(' DX MUST = DY')
      STOP
297   WRITE(IW,291) IFILE
291   FORMAT(3X,A12,' NOT FOUND OR NOT BINARY.  TRY AGAIN. ')
      GO TO 52
298   WRITE(IW,291) SFILE
      GO TO 56
299   WRITE(IW,391) COMMAND
391   FORMAT(' FILE NOT FOUND OR NOT ASCII.  TRY AGAIN.')
      GO TO 1
300   RETURN
      END
C
C THIS SUBROUTINE READS PARAMETERS FROM THE COMMAND FILE IN NAMELIST
C FORMAT
C
      SUBROUTINE GETPARMS(ICMD,IW)
      CHARACTER*56 TITLE
      CHARACTER*50 IFILE,OFILE,SFILE
      COMMON/L2DPARMS/FLEVEL,CONVF,ZLEVEL,IFILT,W1,W2,
     1IDVAL,NADD,NTERMS,IFILE,OFILE,SFILE,TITLE,ITER
      NAMELIST/PARMS/FLEVEL,CONVF,ZLEVEL,NTERMS,
     1IFILT,W1,W2,IDVAL,NADD,IFILE,OFILE,SFILE,TITLE,ITER
      READ(ICMD,PARMS,END=10)
      RETURN
10    WRITE(IW,800)
800   FORMAT(' ODD EOF DETECTED IN CMD FILE.')
      STOP
      END
C ********************************************************************
C      THIS SUBROUTINE RECOMMENDS A REFERENCE LEVEL FOR THE
C       LEVEL-TO-DRAPE OPERATION.
C **********************************************************************
      SUBROUTINE REFLVL(IS,RLEVEL,NROW,NCOL,DX,IW,CONVF,Z)
      DIMENSION Z(NCOL),ID(14),PGM(2)
      DATA ZMAX/-1.E38/,ZMIN/1.E38/
C
C  CALCULATE MIN, MAX AND MEAN OF THE DRAPING SURFACE
C
      SUMZ=0.0
      EN=0.0
      DO 20 J=1,NROW
      READ(11,END=80,ERR=70) DUM,Z
      DO 10 I=1,NCOL
C CONVERT SURF. FILE TO Z POSITIVE DOWN AND INTO UNITS OF DX
C CALCULATE MEAN AND STD DEV
      IF(Z(I).GE.1.E+38) GO TO 10
      Z(I)=Z(I)*CONVF
      SUMZ=SUMZ+Z(I)
      IF(Z(I).GT.ZMAX) ZMAX=Z(I)
      IF(Z(I).LT.ZMIN) ZMIN=Z(I)
      EN=EN+1.0
10    CONTINUE
      IF(J.EQ.1.AND.EN.EQ.0.0) GO TO 60
20    CONTINUE
      REWIND IS
      READ(IS) ID,PGM,NCOL,NROW,NZ,XO,DX,YO,DY
      ZMEAN=SUMZ/EN
C
C  IDEALLY ZMAX-RLEVEL SHOULD BE <= .9*DX (DISTANCE METHOD WILL NEED TO
C  APPROXIMATE DOWNWARD) AND RLEVEL-ZMIN <= 1.5*DX (DISTANCE UPWARD).
C  TRY TO GET CLOSE TO THIS RLEVEL.  USE THE MEAN IF CONDITIONS ARE
C  SATISFIED WITH RLEVEL=ZMEAN.  ROUND OFF RLEVEL.
C
      RLEVEL=ZMEAN
      IF(ZMAX-RLEVEL.LE.0.9*DX.AND.RLEVEL-ZMIN.LE.1.5*DX) GO TO 50
      RTEST1=ZMAX-.9*DX
      RTEST2=ZMIN+1.5*DX
      RLEVEL=(RTEST1+RTEST2)*.5
      IF(ZMAX-RLEVEL.LE.1.2*DX.AND.RLEVEL-ZMIN.LE.2.0*DX) GO TO 50
      WRITE(IW,800)
800   FORMAT(/' **WARNING: RELIEF OF DRAPING SURFACE IS GREATER THAN
     1 DX.',/,' THIS COULD CAUSE THE METHOD TO DIVERGE.  TRY
     2 A COARSER DX?')
C  ROUND OFF RLEVEL
50    IF(RLEVEL.EQ.0.0) GO TO 55
      NSIGN=RLEVEL/ABS(RLEVEL)
      NTEST=2-IFIX(ALOG10(ABS(RLEVEL)))
      TEST=10.E+0**FLOAT(NTEST)
      RTEST=FLOAT(NINT(RLEVEL*TEST))
      RLEVEL=NSIGN*RTEST/TEST
C CHANGE UNITS OF RLEVEL BACK AND RETURN
55    RLEVEL=RLEVEL/CONVF
      RETURN
60    WRITE(IW,801)
801   FORMAT(' NO DATA IN ENTIRE FIRST ROW OF SURFACE FILE.  CAN''T
     1 HANDLE IT.')
      STOP
70    WRITE(IW,802)
802   FORMAT(' ERROR IN READ OF SURFACE FILE.')
      STOP
80    WRITE(IS,803)
803   FORMAT(' UNEXPECTED EOF IN SURFACE FILE.')
      STOP
      END
C***********************************************************************
C  SUBROUTINE 'L2D' REMOVES FLAGGED VALUES FROM GRID, TRANSFORMS
C  DATA, CALLS ROUTINES TO APPLY FILTER,  PERFORMS LEVEL-TO-DRAPE
C  TAYLOR'S SERIES EXPANSION, AND WRITES RESULTING GRID.
C  NOTE THAT BOTH INPUT DATA FILES AND OUTPUT FILE MUST BE OPEN AND
C  HAVE HAD HEADERS READ BEFORE CALLING THIS SUBROUTINE.  THEY WILL
C  REMAIN OPEN UPON RETURN.
C  GUTS OF THIS SUBROUTINE LIFTED FROM PROGRAM FFTFIL BY THOMAS G.
C  HILDENBRAND, 1983, USGS OPEN-FILE REPORT 83-237.
C     F     =     INPUT DATA
C     G     =     STORAGE FOR 1ST VERTICAL DERIVATIVE OF F
C     H     =     STORAGE FOR 2ND VERTICAL DERIVATIVE OF F
C     WORK  =     WORK ARRAY FOR SUBROUTINE SFFTMG
C     N1    =     NUMBER OF ROWS
C     N2    =     NUMBER OF COLS
C     ID2   =     2 * MAX(N1,N2)
C     NRI   =
C     Z     =     SURFACE DATA
C     NXA   =
C
C***********************************************************************
      SUBROUTINE L2D(F,G,H,WORK,ID2,NRI,N1,N2,Z,NXA)
      DIMENSION F(2,N2,NRI),G(2,N2,NRI),H(2,N2,NRI),WORK(ID2),
     1JFLAG(2,20),Z(N2)
      CHARACTER*50 IFILE,OFILE,SFILE,COMMAND
      CHARACTER*56 TITLE
      COMMON /L2DPARMS/FLEVEL,CONVF,RLEVEL,IFILT,W1,W2,IDVAL,NADD,
     1NTERMS,IFILE,OFILE,SFILE,TITLE,ITER
      COMMON /L2DINFO/P1,P2,NZ,YO,XO,DX,DY,NY,NX
      COMMON /UNITS/IW,IR,IN,IOUT,ICMD,IS,IUNITA,IUNITB,IUNITC,IF,
     1IUNITG,IUNITH
      DATA PI2/6.28318531/,DVAL/'FFFF7FFF'X/
      N22=2*N2
      IF (IFILT.EQ.0)GO TO 20
      IF (W1.NE.0.0)GO TO 5
      F4=1.0E+30
      GO TO 7
    5 F4=1./W1
    7 IF (W2.NE.0.0)GO TO 8
      F3=1.0E+30
      GO TO 10
    8 F3=1./W2
   10 IF (F3.NE.F4)DENOM=1./(F4-F3)
C  COMPUTE BEGINNING AND END COLUMN & ROW IN ORIGINAL GRID.
   20 DO 30 I=1,2
      DO 30 J=1,N2
   30 F(2,J,I)=0.0
      NAP1=NADD+1
      N2A=N2-NADD
      NXAP1=NXA*0.5+1
      FXAP1=FLOAT(NXA)/2.+1.
      IF (ABS(FXAP1-FLOAT(NXAP1)).GT.0.0001)NXAP1=NXAP1+1
      N1A=NXAP1-1+NX
C  REMOVE FLAGGED VALUES AND ADD '2*NADD' ROWS & COLUMNS TO
C  REDUCE THE EFFECT OF GIBBS PHENOMENA.
      OPEN (IUNITA,ACCESS='DIRECT',STATUS='NEW',FORM='UNFORMATTED',
     1FILE='SLAVE1.TMP',RECL=N22,MAXREC=N1,
     1ORGANIZATION='RELATIVE',RECORDTYPE='FIXED')
      IF (IDVAL.NE.0)OPEN (IF,STATUS='NEW',FORM='UNFORMATTED',
     1FILE='FLAG.LOC')
      IRR=1
      IWR=0
      JR=0
   40 JR=JR+1
      IF (IDVAL.EQ.0)GO TO 310
C   IDVAL=1: REMOVAL OF FLAGGED VALUES, CREATE FILE FLAG.LOC
      IF (JR.GT.1)GO TO 140
C   READ 1ST ROW & REMOVE FLAGGED VALUES.
      READ (IN)DUM,(F(1,I,1),I=NAP1,N2A)
      READ (IN)DUM,(F(1,I,2),I=NAP1,N2A)
      NFLAG=1
      I=NAP1
   50 JFLAG(1,1)=NAP1
      IF (F(1,NAP1,1).LE.1.0E38)GO TO 90
      DO 60 I=NAP1,N2A
   60 IF (F(1,I,1).LE.1.0E38)GO TO 70
      GO TO 1270
   70 JFLAG(1,1)=I
      AVAL=F(1,I,1)
      DO 80 II=NAP1,I-1
   80 F(1,II,1)=AVAL
   90 IF (I.GE.N2A)GO TO 100
      DO 100 II=I+1,N2A
      IF (F(1,II,1).LE.1.0E38)GO TO 100
      JFLAG(2,NFLAG)=II-1
      GO TO 110
  100 CONTINUE
      JFLAG(2,NFLAG)=N2A
      WRITE (IF)NFLAG,(JFLAG(1,J),JFLAG(2,J),J=1,NFLAG)
      GO TO 350
  110 AVAL=F(1,II-1,1)
      DO 120 I=II,N2A
      IF (F(1,I,1).LE.1.0E38)GO TO 130
  120 F(1,I,1)=AVAL
      WRITE (IF)NFLAG,(JFLAG(1,J),JFLAG(2,J),J=1,NFLAG)
      GO TO 350
  130 F(1,I-1,1)=(F(1,I-1,1)+F(1,I,1))*0.5
      NFLAG=NFLAG+1
      JFLAG(1,NFLAG)=I
C   1ST ROW COMPLETED: NOW EXTEND GRID & WRITE TO DISK.
      GO TO 90
C   REMOVE FLAGGED VALUES FROM REMAINING ROWS.
  140 IF (JR.GE.NX)GO TO 150
      READ (IN)DUM,(F(1,I,3),I=NAP1,N2A)
      GO TO 170
  150 DO 160 I=NAP1,N2A
  160 F(1,I,3)=DVAL
  170 NFLAG=1
      I=NAP1
      JFLAG(1,1)=NAP1
      IF (F(1,NAP1,2).LE.1.0E38)GO TO 220
      DO 180 I=NAP1,N2A
  180 IF (F(1,I,2).LE.1.0E38)GO TO 190
      I=NAP1
      JFLAG(1,1)=N2A+1
      IF (F(1,NAP1,3).GE.1.0E38)F(1,NAP1,2)=F(1,NAP1,1)
      IF (F(1,NAP1,3).LE.1.0E38)F(1,NAP1,2)=(F(1,NAP1,1)+
     1F(1,NAP1,3))*0.5
      GO TO 220
  190 JFLAG(1,1)=I
      II=I
      DO 210 IREV=NAP1,I-1
      II=II-1
      IF (F(1,II,3).GE.1.0E38)GO TO 200
      F(1,II,2)=(F(1,II,1)+F(1,II+1,2)+F(1,II,3))*0.3333333
      GO TO 210
  200 F(1,II,2)=(F(1,II,1)+F(1,II+1,2))*0.5
  210 CONTINUE
  220 IF (I.GE.N2A)GO TO 230
      DO 230 II=I+1,N2A
      IF (F(1,II,2).LE.1.0E38)GO TO 230
      JFLAG(2,NFLAG)=II-1
      GO TO 240
  230 CONTINUE
      JFLAG(2,NFLAG)=N2A
      WRITE (IF)NFLAG,(JFLAG(1,J),JFLAG(2,J),J=1,NFLAG)
      GO TO 390
  240 DO 290 I=II,N2A
      IF (I.EQ.N2A)GO TO 250
      IF (F(1,I+1,2).LE.1.0E38)GO TO 270
  250 IF (F(1,I,3).GE.1.0E38)GO TO 260
      F(1,I,2)=(F(1,I,1)+F(1,I-1,2)+F(1,I,3))*0.3333333
      GO TO 290
  260 F(1,I,2)=(F(1,I,1)+F(1,I-1,2))*0.5
      GO TO 290
  270 IF (F(1,I,3).GE.1.0E38)GO TO 280
      F(1,I,2)=(F(1,I,1)+F(1,I-1,2)+F(1,I+1,2)+F(1,I,3))*0.25
      GO TO 300
  280 F(1,I,2)=(F(1,I,1)+F(1,I-1,2)+F(1,I+1,2))*0.3333333
      GO TO 300
  290 CONTINUE
      WRITE (IF)NFLAG,(JFLAG(1,J),JFLAG(2,J),J=1,NFLAG)
      GO TO 390
  300 NFLAG=NFLAG+1
      I=I+1
      JFLAG(1,NFLAG)=I
      GO TO 220
C   IDVAL.EQ.0:  CHECK INPUT DATA.
  310 IF (JR.NE.1)IRR=2
      READ (IN)DUM,(F(1,I,IRR),I=NAP1,N2A)
      DO 320 II=NAP1,N2A
  320 IF (F(1,II,IRR).GE.1.0E38)GO TO 330
      IF (JR.NE.1)GO TO 390
      GO TO 350
  330 WRITE (IW,340)JR,II
  340 FORMAT (' #FLAGGED VALUE ROW=',I4,3X,'COL=',I4,' MAY BE MORE?')
      CLOSE (IN)
      CLOSE (IUNITA)
      RETURN
C   EXTEND ROW & WRITE TO DISK.
  350 DO 360 I=1,NAP1
  360 F(1,I,1)=F(1,NAP1,1)
      DO 370 I=N2A,N2
  370 F(1,I,1)=F(1,N2A,1)
  380 IWR=IWR+1
      CALL SWRDA(IUNITA,IWR,F,N22)
      IF (IWR.EQ.NXAP1)GO TO 40
      GO TO 380
  390 DO 400 I=1,NAP1
  400 F(1,I,2)=F(1,NAP1,2)
      DO 410 I=N2A,N2
  410 F(1,I,2)=F(1,N2A,2)
      IWR=IWR+1
      CALL SWRDA(IUNITA,IWR,F(1,1,2),N22)
      IF (IWR.GE.N1A)GO TO 430
      IF (IDVAL.LT.1)GO TO 40
      DO 420 I=NAP1,N2A
      F(1,I,1)=F(1,I,2)
  420 F(1,I,2)=F(1,I,3)
      GO TO 40
  430 IF (IWR.GE.N1)GO TO 450
  440 IWR=IWR+1
      CALL SWRDA(IUNITA,IWR,F(1,1,2),N22)
      IF (IWR.LT.N1)GO TO 440
  450 IF (IDVAL.NE.0)CLOSE (IF)
  460 FORMAT (I5)
C  OBTAIN COMPLEX FOURIER COEFFICIENTS (F.F.T. F).
  590 OPEN (IUNITB,ACCESS='DIRECT',STATUS='NEW',FORM='UNFORMATTED',
     1FILE='SLAVE2.TMP',RECL=N22,MAXREC=N1
     2,ORGANIZATION='RELATIVE',RECORDTYPE='FIXED')
C
C    TRANSFORM DATA
C
  980 ISIGN=-1
      CALL SFFTMG(IUNITA,IUNITB,N2,N1,F,NRI,ISIGN,WORK)
C  F ARRAY NOW HOLDS COMPLEX FOURIER COEFFICIENTS.
C  START FILTERING
 1000 GO TO (1005,1003,1001),NTERMS
 1001 OPEN (IUNITH,ACCESS='DIRECT',STATUS='NEW',FORM='UNFORMATTED',
     1FILE='SLAVE4.TMP',RECL=N22,MAXREC=N1,ORGANIZATION='RELATIVE',
     2RECORDTYPE='FIXED')
 1003 OPEN (IUNITG,ACCESS='DIRECT',STATUS='NEW',FORM='UNFORMATTED',
     1FILE='SLAVE3.TMP',RECL=N22,MAXREC=N1,ORGANIZATION='RELATIVE',
     2RECORDTYPE='FIXED')
 1005 D=(FLEVEL-RLEVEL)*CONVF
      CD=PI2*D
      C1=PI2
      N1P1=N1+1
      N2P1=N2+1
      NNH1=(N1/2)+1
      NNH2=(N2/2)+1
C  OPERATING ON F FOURIER COEFFICIENTS.
      RN1=1./(FLOAT(N1)*DX)
      RN2=1./(FLOAT(N2)*DY)
      JR=0
      JMR=N1P1
      IGO=1
      IF (IFILT.EQ.0)GO TO 1010
      IGO=2
      IF (NTERMS.EQ.1.AND.IFILT.LE.1)IGO=3
      IF (NTERMS.EQ.1.AND.IFILT.GE.2)IGO=4
      DUMC=1.0
      DUMD=1.0
      IF (IFILT.LE.2)DUMD=0.0
      IF (IFILT.GE.2)DUMC=0.0
 1010 JR=JR+1
      IF (JR.GT.NNH1)GO TO 1110
      JJ=JR-1
      X=FLOAT(JJ)*RN1
      XSQ=X*X
      CALL SRDDA(IUNITA,JR,F,N22)
      DO 1090 I=1,N2
      II=I-1
      IF (I.GT.NNH2)II=-(N2P1-I)
      Y=FLOAT(II)*RN2
      XYSQ=XSQ+Y*Y
      W=SQRT(XYSQ)
      FR=F(1,I,1)
      FI=F(2,I,1)
      GO TO (1020,1030,1060,1070),IGO
C MODIFY COEFS, NO BANDPASS FILTERING
 1020 E=EXP(CD*W)
      E1=E*C1*W
      E2=E1*C1*W
      F(1,I,1)=FR*E
      F(2,I,1)=FI*E
      G(1,I,1)=FR*E1
      G(2,I,1)=FI*E1
      H(1,I,1)=FR*E2
      H(2,I,1)=FI*E2
      GO TO 1090
C  MODIFY COEFS WHILE BANDPASS FILTERING, DUMD AND DUMC ARE SWITCHES TO
C  FILTER OR NOT TO FILTER DEPENDING ON IFILT
C     DUMD=1.0, NO FILTER OF DERIVS; DUMC=1.0, NO FILTER OF CONTINUATION
 1030 IF (W.LE.F4)GO TO 1050
      E=EXP(CD*W)
      F(1,I,1)=FR*E*DUMC
      F(2,I,1)=FI*E*DUMC
      E1=E*C1*W*DUMD
      E2=E1*C1*W
      G(1,I,1)=FR*E1
      G(2,I,1)=FI*E1
      H(1,I,1)=FR*E2
      H(2,I,1)=FI*E2
      GO TO 1090
 1050 FACTOR=1.0
      IF (W.GT.F3)FACTOR=(F4-W)*DENOM
      E=EXP(CD*W)
      FACTORD=(1.0-DUMD)*FACTOR+DUMD
      FACTORC=(1.-DUMC)*FACTOR+DUMC
      E1=E*C1*W*FACTORD
      E2=E1*C1*W
      F(1,I,1)=FR*E*FACTORC
      F(2,I,1)=FI*E*FACTORC
      G(1,I,1)=FR*E1
      G(2,I,1)=FI*E1
      H(1,I,1)=FR*E2
      H(2,I,1)=FI*E2
      GO TO 1090
C CONTINUATION ONLY (DRAPE-TO-LEVEL), NO BANDPASS FILTERING
 1060 E=EXP(CD*W)
      F(1,I,1)=FR*E
      F(2,I,1)=FI*E
      GO TO 1090
C  CONTINUATION ONLY WITH BANDPASS FILTERING
 1070 IF (W.LE.F4)GO TO 1075
      E=EXP(CD*W)
      F(1,I,1)=0.0
      F(2,I,1)=0.0
      GO TO 1090
 1075 FACTOR=1.0
      IF (W.GT.F3)FACTOR=(F4-W)*DENOM
      E=EXP(CD*W)*FACTOR
      F(1,I,1)=FR*E
      F(2,I,1)=FI*E
 1090 CONTINUE
      CALL SWRDA(IUNITA,JR,F,N22)
      GO TO (1095,1093,1092),NTERMS
 1092 CALL SWRDA(IUNITH,JR,H,N22)
 1093 CALL SWRDA(IUNITG,JR,G,N22)
 1095 IF (JR.EQ.1)GO TO 1010
      JMR=JMR-1
      IF (JR.EQ.JMR)GO TO 1110
      F(2,1,1)=-F(2,1,1)
      G(2,1,1)=-G(2,1,1)
      H(2,1,1)=-H(2,1,1)
      IWR=N2P1
      DO 1100 I=2,NNH2
      IWR=IWR-1
      X=F(2,I,1)
      Y=F(1,I,1)
      F(2,I,1)=-F(2,IWR,1)
      F(1,I,1)=F(1,IWR,1)
      F(1,IWR,1)=Y
      F(2,IWR,1)=-X
      X=G(2,I,1)
      Y=G(1,I,1)
      G(2,I,1)=-G(2,IWR,1)
      G(1,I,1)=G(1,IWR,1)
      G(1,IWR,1)=Y
      G(2,IWR,1)=-X
      X=H(2,I,1)
      Y=H(1,I,1)
      H(2,I,1)=-H(2,IWR,1)
      H(1,I,1)=H(1,IWR,1)
      H(1,IWR,1)=Y
      H(2,IWR,1)=-X
 1100 CONTINUE
      CALL SWRDA(IUNITA,JMR,F,N22)
      GO TO (1010,1105,1102),NTERMS
 1102 CALL SWRDA(IUNITH,JMR,H,N22)
 1105 CALL SWRDA(IUNITG,JMR,G,N22)
      GO TO 1010
C  F ARRAY NOW HOLDS COMPLEX COEFFICIENTS OPERATED ON BY FILTER
C  COMPUTE DESIRED ANOMALY = INV. F.F.T. OF F
 1110 ISIGN=1
      CALL SFFTMG(IUNITA,IUNITB,N2,N1,F,NRI,ISIGN,WORK)
      GO TO (1120,1115,1113),NTERMS
 1113 CALL SFFTMG(IUNITH,IUNITB,N2,N1,H,NRI,ISIGN,WORK)
 1115 CALL SFFTMG(IUNITG,IUNITB,N2,N1,G,NRI,ISIGN,WORK)
C  F ARRAY NOW HOLDS COMPUTED ANOMALY ON DATUM = Z.
C  DO LEVEL TO DRAPE OPERATION AND WRITE TO OUTPUT GRID
 1120 IF (IDVAL.NE.0)OPEN (IF,STATUS='OLD',FORM=
     1'UNFORMATTED',FILE='FLAG.LOC')
      AREA=1./FLOAT(N1*N2)
      DUM=0.0
      IWR=0
      DUMH=1.0
      IF (NTERMS.EQ.2)DUMH=0.0
      IF (NTERMS.EQ.1)GO TO 1215
      DO 1210 JR=NXAP1,N1A
      IWR=IWR+1
      CALL SRDDA(IUNITA,JR,F,N22)
      GO TO (300,1124,1123),NTERMS
 1123 CALL SRDDA(IUNITH,JR,H,N22)
 1124 CALL SRDDA(IUNITG,JR,G,N22)
      READ (IS)DUM,(Z(K),K=1,NY)
      IF (IDVAL.EQ.0)GO TO 1170
      READ (IF)NFLAG,(JFLAG(1,J),JFLAG(2,J),J=1,NFLAG)
      J=1
      IK=0
C   RESTORE SHAPE OF ORIGINAL GRID (I.E. INSERT FLAGGED VALUES).
      DO 1160 I=NAP1,N2A
      IK=IK+1
      IF (I.LT.JFLAG(1,J))GO TO 1130
      IF (I.EQ.JFLAG(2,J))GO TO 1140
      GO TO 1150
 1130 F(1,I,1)=DVAL
      GO TO 1160
 1140 J=J+1
      IF (J.LE.NFLAG)GO TO 1150
      J=J-1
      JFLAG(1,J)=1024
C  CONVERT Z(IK) TO UNITS OF GRID INTERVAL, POSITIVE DOWN, AND
C  INTO UNITS OF DISTANCE FROM REFERENCE LEVEL
 1150 IF (Z(IK).GE.1.E+38)GO TO 1155
      Z(IK)=(RLEVEL-Z(IK))*CONVF
C  TAYLOR'S SERIES EXPANSION.  DUMH DUMMIES OUT THE 2ND DERIV TERM
C    DEPENDING ON NTERMS
      F(1,I,1)=F(1,I,1)+Z(IK)*G(1,I,1)+Z(IK)*Z(IK)*H(1,I,1)*DUMH
      F(1,I,1)=F(1,I,1)*AREA
      GO TO 1160
 1155 F(1,I,1)=DVAL
 1160 CONTINUE
      GO TO 1190
 1170 IK=0
      DO 1180 I=NAP1,N2A
      IK=IK+1
      IF (Z(IK).GE.1.E+38)GO TO 1175
      Z(IK)=(RLEVEL-Z(IK))*CONVF
      F(1,I,1)=F(1,I,1)+Z(IK)*G(1,I,1)+Z(IK)*Z(IK)*H(1,I,1)*DUMH
      F(1,I,1)=F(1,I,1)*AREA
      GO TO 1180
 1175 F(1,I,1)=DVAL
 1180 CONTINUE
 1190 WRITE (IOUT)DUM,(F(1,I,1),I=NAP1,N2A)
 1210 CONTINUE
      GO TO 1250
C RESTORE SHAPE OF GRID- CONTINUATION ONLY OPERATION
 1215 DO 1247 JR=NXAP1,N1A
      IWR=IWR+1
      CALL SRDDA(IUNITA,JR,F,N22)
      IF (IDVAL.EQ.0)GO TO 1235
      READ (IF)NFLAG,(JFLAG(1,J),JFLAG(2,J),J=1,NFLAG)
      J=1
      DO 1230 I=NAP1,N2A
      IF (I.LT.JFLAG(1,J))GO TO 1220
      IF (I.EQ.JFLAG(2,J))GO TO 1225
      GO TO 1227
 1220 F(1,I,1)=DVAL
      GO TO 1230
 1225 J=J+1
      IF (J.LE.NFLAG)GO TO 1227
      J=J-1
      JFLAG(1,J)=1024
 1227 F(1,I,1)=F(1,I,1)*AREA
 1230 CONTINUE
      GO TO 1245
 1235 DO 1240 I=NAP1,N2A
      F(1,I,1)=F(1,I,1)*AREA
 1240 CONTINUE
 1245 WRITE (IOUT)DUM,(F(1,I,1),I=NAP1,N2A)
 1247 CONTINUE
 1250 IF (IDVAL.EQ.1)CLOSE (IF,STATUS='DELETE')
      CLOSE (IUNITC,STATUS='DELETE')
      CLOSE (IUNITA,STATUS='DELETE')
      CLOSE (IUNITB,STATUS='DELETE')
      GO TO (1290,1265,1263),NTERMS
 1263 CLOSE (IUNITH,STATUS='DELETE')
 1265 CLOSE (IUNITG,STATUS='DELETE')
      GO TO 1290
 1270 WRITE (IW,1280)
 1280 FORMAT (' #ENTIRE FIRST ROW HAS NO DATA--PROGRAM ABORTED')
      CLOSE (IN)
      CLOSE (IUNITA,STATUS='DELETE')
      IF (IDVAL.GT.0)CLOSE (IF,STATUS='DELETE')
 1290 RETURN
      END
C ********************************************************************
      SUBROUTINE EXTEND(NADD,N1,N2,ID2,LNRI,NRI,NXA)
C  THIS SUBROUTINE EXTENDS THE COLS OF GRID IF NADD NOT 0 AND FINDS THE
C  BEST NO. OF ROWS TO USE IN THE FFT ROUTINES.  LIFTED FROM PROGRAM
C  FFTFIL BY THOMAS G. HILDENBRAND.
C **********************************************************************
      COMMON /L2DINFO/P1,P2,NZ,YO,XO,DX,DY,NCOL,NROW
      N1=NROW
      N2=NCOL
      L=LNRI+1
      LNRI21=LNRI/2+1
C  SET NO. OF ROWS FOR FFT: NEED M=L*2**K, M.GT. OR .EQ. NX+2*NADD.
C  M=NO. OF ROWS, L=NO. FROM 9-16, K=INTERGER.
      N1=N1+2*NADD
  190 L=L-1
      IF (L.LT.LNRI21)GO TO 260
      MR=N1/L+0.0000001
      K=1
      IDIV=2
  200 IQUOT=MR/IDIV+0.0000001
      IF (IQUOT.LT.IDIV)GO TO 210
      K=K+1
      MR=IQUOT
      GO TO 200
  210 K=K+1
      M=L*2**K
  220 MTEST=L*2**(K-1)
      IF (MTEST.LT.N1)GO TO 230
      K=K-1
      M=MTEST
      IF (K.EQ.0)GO TO 230
      GO TO 220
  230 LNXA=M-N1
      IF (L.NE.LNRI)GO TO 250
      NXA16=LNXA
  240 NRI=L
      NXA=LNXA
      GO TO 190
  250 IF (LNXA.GE.NXA)GO TO 190
      GO TO 240
  260 N1=N1+NXA
C  CHECK TO SEE IF ROW BLOCK SIZE OF 16 WILL BE MORE EFFICIENT
      N116=N1-NXA+NXA16
      NTEST=0.9*N116
      IF (NTEST.GT.N1.OR.N116.GT.1024)GO TO 270
      N1=N1-NXA+NXA16
      NXA=NXA16
      NRI=16
  270 N2=N2+2*NADD
      IF (N1.GT.1024.OR.N2.GT.1024)GO TO 320
      NXA=NXA+2*NADD
      ID2=N1
      IF (N2.GT.N1)ID2=N2
      ID2=2*ID2
      GO TO 300
  320 WRITE (IW,330)NROW,NCOL,N1,N2
  330 FORMAT (' NO. OF EXTENDED ROWS OR COLS EXCEEDS 1024:',/,' INPUT
     1NO. OF ROWS AND COLS=',2I4,/,'NO. OF ROWS AND COLS REQUIRED FOR
     2 FILTERING=',2I4,/)
  340 STOP
  300 RETURN
      END
C***********************************************************************
      SUBROUTINE SFFTMG(UNITA,UNITB,N,M,F,L,ISIGN,WORK)
C     PERFORMS 2-DIMENSIONAL FAST FOURIER TRANSFORM USING DISK
C     ARGUMENTS -
C       UNITA - UNIT NUMBER OF OPEN RANDOM ACCESS DISK FILE CONTAINING
C               ONE COMPLEX DATA ROW PER RECORD.
C               THIS FILE IS USED FOR INPUT TO FFTMG AND OUTPUT
C               OF THE TRANSFORM FROM FFTMG.
C       UNITB - UNIT NUMBER OF OPEN RANDOM ACCESS WORK FILE
C               THE SAME AS THE FILE OF UNITA.
C       N     - NUMBER OF Y COLUMNS = LENGTH OF X ROWS.
C       M     - NUMBER OF X ROWS = LENGTH OF Y COLUMNS,
C               MUST BE OF THE FORM L*2**K (K INTEGER).
C       F     - COMPLEX WORK ARRAY FOR CORE MANIPULATIONS.
C       L     - NUMBER OF X ROWS WHICH CAN BE HELD IN F.
C       ISIGN - +1 OR -1 FOR FORWARD OR REVERSE TRANSFORM.
C       WORK  - WORK ARRAY FOR IN-CORE TRANSFORM ROUTINE SFOURT.
C               WORK MUST BE COMPLEX, OF LENGTH MAX(N,L).
C   DEVELOPED AND CODED BY RAY WATTS--USGS,DENVER
C   NOTE: RANDIN,RANDOU,INSET, AND OUTSET ARE ENTRY POINTS
C   IN SUBROUTINE SDUMMY
C***********************************************************************
      INTEGER UNITA,UNITB,OUTPUT,NDIM(2)
      COMPLEX F(N,L),TWIDLE,TWIDLF,W,WORK(1)
      NFOURT=2*N*L
      NWORK=2*N
      IF(L.GT.N)NWORK=2*L
C     **********************************************************
C     *    THE NUMBER OF DISK ACCESSES (READS+WRITES) USED BY  *
C     *    THIS PROGRAM IS M*(4*LOG2(M/L)+3).                  *
C     **********************************************************
C     --------------------------------------------------------
C     CHECK ARGUMENTS
      IF(L*(M/L).NE.M)STOP 'L NOT A FACTOR OF M IN FFTMG'
      ML=M/L
      MT=1
   10 IF(MT-ML)20,40,30
   20 MT=2*MT
      GO TO 10
   30 STOP 'M/L NOT A POWER OF 2 IN FFTMG'
   40 CONTINUE
C     ------------------------------------------------------------
C     -------------------------------------------------------------
C     START PROCESSING
C     SKIP DECIMATION IF L .EQ. M
      IF(L.EQ.M)GO TO 80
C     SET STARTING INPUT AND OUTPUT UNITS
      INPUT=UNITA
      OUTPUT=UNITB
C     SET STARTING BLOCKSIZE FOR DECIMATION IN INPUT DOMAIN
      IBLOCK=M
C     -------------------------------------------------------
C     DECIMATE THIS BLOCKSIZE
C     SET UP SUB-BLOCK SIZE
   50 ISUB=IBLOCK/2
C     SET UP FOR RANDOM I/0
      CALL INSET(INPUT)
      CALL OUTSET(OUTPUT)
C     FOR EACH BLOCK OF THIS SIZE
      DO 60 IBLST=1,M,IBLOCK
C     COPY EVEN MEMBERS INTO FIRST SUB-BLOCK, ODD MEMBERS INTO SECOND
      DO 60 I=1,ISUB
      CALL RANDIN(N,F,2*(I-1)+IBLST)
      CALL RANDOU(N,F,I-1+IBLST)
      CALL RANDIN(N,F,2*I-1+IBLST)
   60 CALL RANDOU(N,F,I-1+IBLST+ISUB)
C     SWAP INPUT/OUTPUT FUNCTIONS
      INPUTT=INPUT
      INPUT=OUTPUT
      OUTPUT=INPUTT
C     NEXT SMALLER BLOCKSIZE
      IBLOCK=ISUB
C     LOOP IF NOT DONE
      IF(IBLOCK.GT.L)GO TO 50
C     DECIMATION COMPLETE
C     ---------------------------------------------------------
C     ---------------------------------------------------------
C     ENSURE THAT DATA SET IS BACK ON UNITA
      IF(INPUT.EQ.UNITA)GO TO 80
      CALL INSET(INPUT)
      CALL OUTSET(OUTPUT)
      DO 70 I=1,M
      CALL RANDIN(N,F,I)
   70 CALL RANDOU(N,F,I)
   80 CONTINUE
C     --------------------------------------------------------
C     START TRANSFORM.
C     SET BLOCK SIZE
      IBLOCK=L
C     SET UP FOR RANDOM I/O
      CALL INSET(UNITA)
      CALL OUTSET(UNITA)
C     RUN THROUGH THE BLOCKS
      DO 100 ISTART=1,M,IBLOCK
      INTO=1
C     RUN THROUGH THE RECORDS IN A BLOCK
      DO 90 IRECRD=ISTART,ISTART+IBLOCK-1
C     READ THE RECORD
      CALL RANDIN(N,F(1,INTO),IRECRD)
C     PROCEED WITH NEXT RECORD IN BLOCK
   90 INTO=INTO+1
C     FOURIER TRANSFORM IN BOTH DIRECTIONS
      NDIM(1)=N
      NDIM(2)=L
      CALL SFOURT(F,NDIM,2,ISIGN,1,WORK,NFOURT,NWORK)
C     ******************************************************
C     WRITE THE BLOCK BACK ONTO DISK
      IFROM=1
      DO 100 IRECRD=ISTART,ISTART+IBLOCK-1
      CALL RANDOU(N,F(1,IFROM),IRECRD)
  100 IFROM=IFROM+1
C     IF L.EQ.M, THE TRANSFORM IS NOW COMPLETE.
      IF(L.EQ.M)RETURN
C     DONE WITH FIRST-STEP PROCESSING
C     ---------------------------------------------------------
C     FINISH THE PROCESSING BY PAIRS OF ROWS, APPLYING TWIDDLE FACTORS
C     SET THE TWIDDLE GENERATOR
      TWIDLF=CEXP(CMPLX(0.,3.141593*ISIGN/(IBLOCK)))
C     RUN THROUGH THE BLOCKS BY PAIRS
  110 DO 140 ISTART=1,M,IBLOCK*2
C     START THE TWIDDLE FACTOR
      TWIDLE=(1.,0.)
C     SCAN THE BLOCK PAIR
      DO 130 IRECRD=ISTART,ISTART+IBLOCK-1
      JRECRD=IRECRD+IBLOCK
      CALL RANDIN(N,F,IRECRD)
      CALL RANDIN(N,F(1,2),JRECRD)
C     COMBINE, USING TWIDDLE FACTOR
      DO 120 I=1,N
      W=TWIDLE*F(I,2)
      F(I,2)=F(I,1)-W
  120 F(I,1)=F(I,1)+W
C     WRITE BACK ONTO DISK
      CALL RANDOU(N,F,IRECRD)
      CALL RANDOU(N,F(1,2),JRECRD)
C     INCREMENT THE TWIDDLE FACTOR
  130 TWIDLE=TWIDLE*TWIDLF
C     GO ON TO NEXT BLOCK PAIR
  140 CONTINUE
C     INCREASE THE BLOCK SIZE, CHANGE TWIDDLE INCREMENT
      IBLOCK=2*IBLOCK
      TWIDLF=CSQRT(TWIDLF)
      IF(ISIGN*AIMAG(TWIDLF).LT.0)TWIDLF=-TWIDLF
C     CHECK IF DONE
      IF(IBLOCK.LT.M)GO TO 110
C     ----------------------------------------------------------
C     DONE PROCESSING
      END
C ********************************************************************
      SUBROUTINE SRDDA(NO,IPOS,DAT,N)
C   THIS SUBROUTINE READS KEYED SEQUENTIAL FILES.
C ********************************************************************
      DIMENSION DAT(N)
      READ(NO'IPOS)DAT
      RETURN
      END
C ********************************************************************
      SUBROUTINE SWRDA(NO,IPOS,DAT,N)
C   THIS SUBROUTINE WRITES KEYED SEQUENTIAL FILES
C ********************************************************************
      DIMENSION DAT(N)
      WRITE(NO'IPOS)DAT
      RETURN
      END
C***********************************************************************
      SUBROUTINE SFOURT(DATA,NN,NDIM,ISIGN,IFORM,WORK,NFOURT,NWORK1)
C  NOTE THAT 'NFOURT' AND 'NWORK1' IN SUBROUTINE CALL STATEMENT ARE
C  REVISED FEATURES OF 'SFOURT'. THEY ARE USED AS DYNAMIC DIMENSIONING
C  FOR THE 'DATA' AND 'WORK' ARRAYS
C     THE COOLEY-TUKEY FAST FOURIER TRANSFORM IN USASI BASIC FORTRAN
C     TRANSFORM(J1,J2,) = SUM(DATA(I1,I2,)*W1**((I1-1)*(J1-1))
C                                 *W2**((I2-1)*(J2-1))*,),
C     WHERE I1 AND J1 RUN FROM 1 TO NN(1) AND W1=EXP(ISIGN*2*PI*
C     SQRT(-1)/NN(1)), ETC.  THERE IS NO LIMIT ON THE DIMENSIONALITY
C     (NUMBER OF SUBSCRIPTS) OF THE DATA ARRAY.  IF AN INVERSE
C     TRANSFORM (ISIGN=+1) IS PERFORMED UPON AN ARRAY OF TRANSFORMED
C     (ISIGN=-1) DATA, THE ORIGINAL DATA WILL REAPPEAR.
C     MULTIPLIED BY NN(1)*NN(2)*,  THE ARRAY OF INPUT DATA MUST BE
C     IN COMPLEX FORMAT.  HOWEVER, IF ALL IMAGINARY PARTS ARE ZERO (I.E.
C     THE DATA ARE DISGUISED REAL) RUNNING TIME IS CUT UP TO FORTY PER-
C     CENT.  (FOR FASTEST TRANSFORM OF REAL DATA, NN(1) SHOULD BE EVEN.)
C     THE TRANSFORM VALUES ARE ALWAYS COMPLEX AND ARE RETURNED IN THE
C     ORIGINAL ARRAY OF DATA, REPLACING THE INPUT DATA.  THE LENGTH
C     OF EACH DIMENSION OF THE DATA ARRAY MAY BE ANY INTEGER.  THE
C     PROGRAM RUNS FASTER ON COMPOSITE INTEGERS THAN ON PRIMES, AND IS
C     PARTICULARLY FAST ON NUMBERS RICH IN FACTORS OF TWO.
C     TIMING IS IN FACT GIVEN BY THE FOLLOWING FORMULA.  LET NTOT BE THE
C     TOTAL NUMBER OF POINTS (REAL OR COMPLEX) IN THE DATA ARRAY, THAT
C     IS, NTOT=NN(1)*NN(2)*...  DECOMPOSE NTOT INTO ITS PRIME FACTORS,
C     SUCH AS 2**K2 * 3**K3 * 5**K5 * ...  LET SUM2 BE THE SUM OF ALL
C     THE FACTORS OF TWO IN NTOT, THAT IS, SUM2 = 2*K2.  LET SUMF BE
C     THE SUM OF ALL OTHER FACTORS OF NTOT, THAT IS, SUMF = 3*K3*5*K5*..
C     THE TIME TAKEN BY A MULTIDIMENSIONAL TRANSFORM ON THESE NTOT DATA
C     IS T = T0 + NTOT*(T1+T2*SUM2+T3*SUMF).  ON THE CDC 3300 (FLOATING
C     POINT ADD TIME = SIX MICROSECONDS), T = 3000 + NTOT*(600+40*SUM2+
C     175*SUMF) MICROSECONDS ON COMPLEX DATA.
C     IMPLEMENTATION OF THE DEFINITION BY SUMMATION WILL RUN IN A TIME
C     PROPORTIONAL TO NTOT*(NN(1)+NN(2)+...).  FOR HIGHLY COMPOSITE NTOT
C     THE SAVINGS OFFERED BY THIS PROGRAM CAN BE DRAMATIC.  A ONE-DIMEN-
C     SIONAL ARRAY 4000 IN LENGTH WILL BE TRANSFORMED IN 4000*(600+
C     40*(2+2+2+2+2)+175*(5+5+5)) = 14.5 SECONDS VERSUS ABOUT 4000*
C     4000*175 = 2800 SECONDS FOR THE STRAIGHTFORWARD TECHNIQUE.
C     THE FAST FOURIER TRANSFORM PLACES THREE RESTRICTIONS UPON THE
C     DATA.
C     1.  THE NUMBER OF INPUT DATA AND THE NUMBER OF TRANSFORM VALUES
C     MUST BE THE SAME.
C     2.  BOTH THE INPUT DATA AND THE TRANSFORM VALUES MUST REPRESENT
C     EQUISPACED POINTS IN THEIR RESPECTIVE DOMAINS OF TIME AND
C     FREQUENCY.  CALLING THESE SPACINGS DELTAT AND DELTAF, IT MUST BE
C     TRUE THAT DELTAF=2*PI/(NN(I)*DELTAT).  OF COURSE, DELTAT NEED NOT
C     BE THE SAME FOR EVERY DIMENSION.
C     3.  CONCEPTUALLY AT LEAST, THE INPUT DATA AND THE TRANSFORM OUTPUT
C     REPRESENT SINGLE CYCLES OF PERIODIC FUNCTIONS.
C     THE CALLING SEQUENCE IS--
C     CALL FOURT(DATA,NN,NDIM,ISIGN,IFORM,WORK)
C     DATA IS THE ARRAY USED TO HOLD THE REAL AND IMAGINARY PARTS
C     OF THE DATA ON INPUT AND THE TRANSFORM VALUES ON OUTPUT.  IT
C     IS A MULTIDIMENSIONAL FLOATING POINT ARRAY, WITH THE REAL AND
C     IMAGINARY PARTS OF A DATUM STORED IMMEDIATELY ADJACENT IN STORAGE
C     (SUCH AS FORTRAN IV PLACES THEM).  NORMAL FORTRAN ORDERING IS
C     EXPECTED, THE FIRST SUBSCRIPT CHANGING FASTEST.  THE DIMENSIONS
C     ARE GIVEN IN THE INTEGER ARRAY NN, OF LENGTH NDIM.  ISIGN IS -1
C     TO INDICATE A FORWARD TRANSFORM (EXPONENTIAL SIGN IS -) AND +1
C     FOR AN INVERSE TRANSFORM (SIGN IS +).  IFORM IS +1 IF THE DATA ARE
C     COMPLEX, 0 IF THE DATA ARE REAL.  IF IT IS 0, THE IMAGINARY
C     PARTS OF THE DATA MUST BE SET TO ZERO.  AS EXPLAINED ABOVE, THE
C     TRANSFORM VALUES ARE ALWAYS COMPLEX AND ARE STORED IN ARRAY DATA.
C     WORK IS AN ARRAY USED FOR WORKING STORAGE.  IT IS FLOATING POINT
C     REAL, ONE DIMENSIONAL OF LENGTH EQUAL TO TWICE THE LARGEST ARRAY
C     DIMENSION NN(I) THAT IS NOT A POWER OF TWO.  IF ALL NN(I) ARE
C     POWERS OF TWO, IT IS NOT NEEDED AND MAY BE REPLACED BY ZERO IN THE
C     CALLING SEQUENCE.  THUS, FOR A ONE-DIMENSIONAL ARRAY, NN(1) ODD,
C     WORK OCCUPIES AS MANY STORAGE LOCATIONS AS DATA.  IF SUPPLIED,
C     WORK MUST NOT BE THE SAME ARRAY AS DATA.  ALL SUBSCRIPTS OF ALL
C     ARRAYS BEGIN AT ONE.
C     EXAMPLE 1.  THREE-DIMENSIONAL FORWARD FOURIER TRANSFORM OF A
C     COMPLEX ARRAY DIMENSIONED 32 BY 25 BY 13 IN FORTRAN IV.
C     DIMENSION DATA(32,25,13),WORK(50),NN(3)
C     COMPLEX DATA
C     DATA NN/32,25,13/
C     DO 1 I=1,32
C     DO 1 J=1,25
C     DO 1 K=1,13
C  1  DATA(I,J,K)=COMPLEX VALUE
C     CALL FOURT(DATA,NN,3,-1,1,WORK)
C     EXAMPLE 2.  ONE-DIMENSIONAL FORWARD TRANSFORM OF A REAL ARRAY OF
C     LENGTH 64 IN FORTRAN II,
C     DIMENSION DATA(2,64)
C     DO 2 I=1,64
C     DATA(1,I)=REAL PART
C  2  DATA(2,I)=0.
C     CALL FOURT(DATA,64,1,-1,0,0)
C     THERE ARE NO ERROR MESSAGES OR ERROR HALTS IN THIS PROGRAM.  THE
C     PROGRAM RETURNS IMMEDIATELY IF NDIM OR ANY NN(I) IS LESS THAN ONE.
C     PROGRAM BY NORMAN BRENNER FROM THE BASIC PROGRAM BY CHARLES
C     RADER,  JUNE 1967.  THE IDEA FOR THE DIGIT REVERSAL WAS
C     SUGGESTED BY RALPH ALTER.
C     THIS IS THE FASTEST AND MOST VERSATILE VERSION OF THE FFT KNOWN
C     TO THE AUTHOR.  A PROGRAM CALLED FOUR2 IS AVAILABLE THAT ALSO
C     PERFORMS THE FAST FOURIER TRANSFORM AND IS WRITTEN IN USASI BASIC
C     FORTRAN.  IT IS ABOUT ONE THIRD AS LONG AND RESTRICTS THE
C     DIMENSIONS OF THE INPUT ARRAY (WHICH MUST BE COMPLEX) TO BE POWERS
C     OF TWO.  ANOTHER PROGRAM, CALLED FOUR1, IS ONE TENTH AS LONG AND
C     RUNS TWO THIRDS AS FAST ON A ONE-DIMENSIONAL COMPLEX ARRAY WHOSE
C     LENGTH IS A POWER OF TWO.
C     REFERENCE--
C     IEEE AUDIO TRANSACTIONS (JUNE 1967), SPECIAL ISSUE ON THE FFT.
C***********************************************************************
      DIMENSION DATA(NFOURT),NN(2),IFACT(32),WORK(NWORK1)
      DATA TWOPI/6.2831853071796/,RTHLF/0.70710678118655/
      IF(NDIM-1)1190,10,10
   10 NTOT=2
      DO 20 IDIM=1,NDIM
      IF(NN(IDIM))1190,1190,20
   20 NTOT=NTOT*NN(IDIM)
C     MAIN LOOP FOR EACH DIMENSION
      NP1=2
      DO 1180 IDIM=1,NDIM
      N=NN(IDIM)
      NP2=NP1*N
      IF(N-1)1190,1170,30
C     IS N A POWER OF TWO AND IF NOT, WHAT ARE ITS FACTORS
   30 M=N
      NTWO=NP1
      IF=1
      IDIV=2
   40 IQUOT=M/IDIV
      IREM=M-IDIV*IQUOT
      IF(IQUOT-IDIV)120,50,50
   50 IF(IREM)70,60,70
   60 NTWO=NTWO+NTWO
      IFACT(IF)=IDIV
      IF=IF+1
      M=IQUOT
      GO TO 40
   70 IDIV=3
      INON2=IF
   80 IQUOT=M/IDIV
      IREM=M-IDIV*IQUOT
      IF(IQUOT-IDIV)140,90,90
   90 IF(IREM)110,100,110
  100 IFACT(IF)=IDIV
      IF=IF+1
      M=IQUOT
      GO TO 80
  110 IDIV=IDIV+2
      GO TO 80
  120 INON2=IF
      IF(IREM)140,130,140
  130 NTWO=NTWO+NTWO
      GO TO 150
  140 IFACT(IF)=M
C     SEPARATE FOUR CASES--
C        1. COMPLEX TRANSFORM OR REAL TRANSFORM FOR THE 4TH, 9TH,ETC.
C           DIMENSIONS.
C        2. REAL TRANSFORM FOR THE 2ND OR 3RD DIMENSION.  METHOD--
C           TRANSFORM HALF THE DATA, SUPPLYING THE OTHER HALF BY CON-
C           JUGATE SYMMETRY.
C        3. REAL TRANSFORM FOR THE 1ST DIMENSION, N ODD.  METHOD--
C           SET THE IMAGINARY PARTS TO ZERO.
C        4. REAL TRANSFORM FOR THE 1ST DIMENSION, N EVEN.  METHOD--
C           TRANSFORM A COMPLEX ARRAY OF LENGTH N/2 WHOSE REAL PARTS
C           ARE THE EVEN NUMBERED REAL VALUES AND WHOSE IMAGINARY PARTS
C           ARE THE ODD NUMBERED REAL VALUES.  SEPARATE AND SUPPLY
C           THE SECOND HALF BY CONJUGATE SYMMETRY.
  150 ICASE=1
      IFMIN=1
      I1RNG=NP1
      IF(IDIM-4)160,210,210
  160 IF(IFORM)170,170,210
  170 ICASE=2
      I1RNG=NP0*(1+NPREV/2)
      IF(IDIM-1)180,180,210
  180 ICASE=3
      I1RNG=NP1
      IF(NTWO-NP1)210,210,190
  190 ICASE=4
      IFMIN=2
      NTWO=NTWO/2
      N=N/2
      NP2=NP2/2
      NTOT=NTOT/2
      I=1
      DO 200 J=1,NTOT
      DATA(J)=DATA(I)
  200 I=I+2
C     SHUFFLE DATA BY BIT REVERSAL, SINCE N=2**K.  AS THE SHUFFLING
C     CAN BE DONE BY SIMPLE INTERCHANGE, NO WORKING ARRAY IS NEEDED
  210 IF(NTWO-NP2)290,220,220
  220 NP2HF=NP2/2
      J=1
      DO 280 I2=1,NP2,NP1
      IF(J-I2)230,250,250
  230 I1MAX=I2+NP1-2
      DO 240 I1=I2,I1MAX,2
      DO 240 I3=I1,NTOT,NP2
      J3=J+I3-I2
      TEMPR=DATA(I3)
      TEMPI=DATA(I3+1)
      DATA(I3)=DATA(J3)
      DATA(I3+1)=DATA(J3+1)
      DATA(J3)=TEMPR
  240 DATA(J3+1)=TEMPI
  250 M=NP2HF
  260 IF(J-M)280,280,270
  270 J=J-M
      M=M/2
      IF(M-NP1)280,260,260
  280 J=J+M
      GO TO 370
C     SHUFFLE DATA BY DIGIT REVERSAL FOR GENERAL N
  290 NWORK=2*N
      DO 360 I1=1,NP1,2
      DO 360 I3=I1,NTOT,NP2
      J=I3
      DO 350 I=1,NWORK,2
      IF(ICASE-3)300,310,300
  300 WORK(I)=DATA(J)
      WORK(I+1)=DATA(J+1)
      GO TO 320
  310 WORK(I)=DATA(J)
      WORK(I+1)=0.
  320 IFP2=NP2
      IF=IFMIN
  330 IFP1=IFP2/IFACT(IF)
      J=J+IFP1
      IF(J-I3-IFP2)350,340,340
  340 J=J-IFP2
      IFP2=IFP1
      IF=IF+1
      IF(IFP2-NP1)350,350,330
  350 CONTINUE
      I2MAX=I3+NP2-NP1
      I=1
      DO 360 I2=I3,I2MAX,NP1
      DATA(I2)=WORK(I)
      DATA(I2+1)=WORK(I+1)
  360 I=I+2
C     MAIN LOOP FOR FACTORS OF TWO.  PERFORM FOURIER TRANSFORMS OF
C     LENGTH FOUR, WITH ONE OF LENGTH TWO IF NEEDED.  THE TWIDDLE FACTOR
C     W=EXP(ISIGN*2*PI*SQRT(-1)*M/(4*MMAX)).  CHECK FOR W=ISIGN*SQRT(-1)
C     AND REPEAT FOR W=W*(1+ISIGN*SQRT(-1))/SQRT(2).
  370 IF(NTWO-NP1)680,680,380
  380 NP1TW=NP1+NP1
      IPAR=NTWO/NP1
  390 IF(IPAR-2)430,410,400
  400 IPAR=IPAR/4
      GO TO 390
  410 DO 420 I1=1,I1RNG,2
      DO 420 K1=I1,NTOT,NP1TW
      K2=K1+NP1
      TEMPR=DATA(K2)
      TEMPI=DATA(K2+1)
      DATA(K2)=DATA(K1)-TEMPR
      DATA(K2+1)=DATA(K1+1)-TEMPI
      DATA(K1)=DATA(K1)+TEMPR
  420 DATA(K1+1)=DATA(K1+1)+TEMPI
  430 MMAX=NP1
  440 IF(MMAX-NTWO/2)450,680,680
  450 LMAX=MAX0(NP1TW,MMAX/2)
      DO 670 L=NP1,LMAX,NP1TW
      M=L
      IF(MMAX-NP1)500,500,460
  460 THETA=-TWOPI*FLOAT(L)/FLOAT(4*MMAX)
      IF(ISIGN)480,470,470
  470 THETA=-THETA
  480 WR=COS(THETA)
      WI=SIN(THETA)
  490 W2R=WR*WR-WI*WI
      W2I=2.*WR*WI
      W3R=W2R*WR-W2I*WI
      W3I=W2R*WI+W2I*WR
  500 DO 630 I1=1,I1RNG,2
      KMIN=I1+IPAR*M
      IF(MMAX-NP1)510,510,520
  510 KMIN=I1
  520 KDIF=IPAR*MMAX
  530 KSTEP=4*KDIF
      IF(KSTEP-NTWO)540,540,630
  540 DO 620 K1=KMIN,NTOT,KSTEP
      K2=K1+KDIF
      K3=K2+KDIF
      K4=K3+KDIF
      IF(MMAX-NP1)550,550,580
  550 U1R=DATA(K1)+DATA(K2)
      U1I=DATA(K1+1)+DATA(K2+1)
      U2R=DATA(K3)+DATA(K4)
      U2I=DATA(K3+1)+DATA(K4+1)
      U3R=DATA(K1)-DATA(K2)
      U3I=DATA(K1+1)-DATA(K2+1)
      IF(ISIGN)560,570,570
  560 U4R=DATA(K3+1)-DATA(K4+1)
      U4I=DATA(K4)-DATA(K3)
      GO TO 610
  570 U4R=DATA(K4+1)-DATA(K3+1)
      U4I=DATA(K3)-DATA(K4)
      GO TO 610
  580 T2R=W2R*DATA(K2)-W2I*DATA(K2+1)
      T2I=W2R*DATA(K2+1)+W2I*DATA(K2)
      T3R=WR*DATA(K3)-WI*DATA(K3+1)
      T3I=WR*DATA(K3+1)+WI*DATA(K3)
      T4R=W3R*DATA(K4)-W3I*DATA(K4+1)
      T4I=W3R*DATA(K4+1)+W3I*DATA(K4)
      U1R=DATA(K1)+T2R
      U1I=DATA(K1+1)+T2I
      U2R=T3R+T4R
      U2I=T3I+T4I
      U3R=DATA(K1)-T2R
      U3I=DATA(K1+1)-T2I
      IF(ISIGN)590,600,600
  590 U4R=T3I-T4I
      U4I=T4R-T3R
      GO TO 610
  600 U4R=T4I-T3I
      U4I=T3R-T4R
  610 DATA(K1)=U1R+U2R
      DATA(K1+1)=U1I+U2I
      DATA(K2)=U3R+U4R
      DATA(K2+1)=U3I+U4I
      DATA(K3)=U1R-U2R
      DATA(K3+1)=U1I-U2I
      DATA(K4)=U3R-U4R
  620 DATA(K4+1)=U3I-U4I
      KDIF=KSTEP
      KMIN=4*(KMIN-I1)+I1
      GO TO 530
  630 CONTINUE
      M=M+LMAX
      IF(M-MMAX)640,640,670
  640 IF(ISIGN)650,660,660
  650 TEMPR=WR
      WR=(WR+WI)*RTHLF
      WI=(WI-TEMPR)*RTHLF
      GO TO 490
  660 TEMPR=WR
      WR=(WR-WI)*RTHLF
      WI=(TEMPR+WI)*RTHLF
      GO TO 490
  670 CONTINUE
      IPAR=3-IPAR
      MMAX=MMAX+MMAX
      GO TO 440
C     MAIN LOOP FOR FACTORS NOT EQUAL TO TWO.  APPLY THE TWIDDLE FACTOR
C     W=EXP(ISIGN*2*PI*SQRT(-1)*(J1-1)*(J2-J1)/(IFP1+IFP2)), THEN
C     PERFORM A FOURIER TRANSFORM OF LENGTH IFACT(IF), MAKING USE OF
C     CONJUGATE SYMMETRIES.
  680 IF(NTWO-NP2)690,900,900
  690 IFP1=NTWO
      IF=INON2
      NP1HF=NP1/2
  700 IFP2=IFACT(IF)*IFP1
      J1MIN=NP1+1
      IF(J1MIN-IFP1)710,710,760
  710 DO 750 J1=J1MIN,IFP1,NP1
      THETA=-TWOPI*FLOAT(J1-1)/FLOAT(IFP2)
      IF(ISIGN)730,720,720
  720 THETA=-THETA
  730 WSTPR=COS(THETA)
      WSTPI=SIN(THETA)
      WR=WSTPR
      WI=WSTPI
      J2MIN=J1+IFP1
      J2MAX=J1+IFP2-IFP1
      DO 750 J2=J2MIN,J2MAX,IFP1
      I1MAX=J2+I1RNG-2
      DO 740 I1=J2,I1MAX,2
      DO 740 J3=I1,NTOT,IFP2
      TEMPR=DATA(J3)
      DATA(J3)=DATA(J3)*WR-DATA(J3+1)*WI
  740 DATA(J3+1)=TEMPR*WI+DATA(J3+1)*WR
      TEMPR=WR
      WR=WR*WSTPR-WI*WSTPI
  750 WI=TEMPR*WSTPI+WI*WSTPR
  760 THETA=-TWOPI/FLOAT(IFACT(IF))
      IF(ISIGN)780,770,770
  770 THETA=-THETA
  780 WSTPR=COS(THETA)
      WSTPI=SIN(THETA)
      J2RNG=IFP1*(1+IFACT(IF)/2)
      DO 890 I1=1,I1RNG,2
      DO 890 I3=I1,NTOT,NP2
      J2MAX=I3+J2RNG-IFP1
      DO 880 J2=I3,J2MAX,IFP1
      J1MAX=J2+IFP1-NP1
      DO 850 J1=J2,J1MAX,NP1
      J3MAX=J1+NP2-IFP2
      DO 850 J3=J1,J3MAX,IFP2
      JMIN=J3-J2+I3
      JMAX=JMIN+IFP2-IFP1
      I=1+(J3-I3)/NP1HF
      IF(J2-I3)790,790,820
  790 SUMR=0.
      SUMI=0.
      DO 810 J=JMIN,JMAX,IFP1
  800 SUMR=SUMR+DATA(J)
  810 SUMI=SUMI+DATA(J+1)
      WORK(I)=SUMR
      WORK(I+1)=SUMI
      GO TO 850
  820 ICONJ=1+(IFP2-2*J2+I3+J3)/NP1HF
      J=JMAX
      SUMR=DATA(J)
      SUMI=DATA(J+1)
      OLDSR=0.
      OLDSI=0.
      J=J-IFP1
  830 TEMPR=SUMR
      TEMPI=SUMI
      SUMR=TWOWR*SUMR-OLDSR+DATA(J)
      SUMI=TWOWR*SUMI-OLDSI+DATA(J+1)
      OLDSR=TEMPR
      OLDSI=TEMPI
      J=J-IFP1
      IF(J-JMIN)840,840,830
  840 TEMPR=WR*SUMR-OLDSR+DATA(J)
      TEMPI=WI*SUMI
      WORK(I)=TEMPR-TEMPI
      WORK(ICONJ)=TEMPR+TEMPI
      TEMPR=WR*SUMI-OLDSI+DATA(J+1)
      TEMPI=WI*SUMR
      WORK(I+1)=TEMPR+TEMPI
      WORK(ICONJ+1)=TEMPR-TEMPI
  850 CONTINUE
      IF(J2-I3)860,860,870
  860 WR=WSTPR
      WI=WSTPI
      GO TO 880
  870 TEMPR=WR
      WR=WR*WSTPR-WI*WSTPI
      WI=TEMPR*WSTPI+WI*WSTPR
  880 TWOWR=WR+WR
      I=1
      I2MAX=I3+NP2-NP1
      DO 890 I2=I3,I2MAX,NP1
      DATA(I2)=WORK(I)
      DATA(I2+1)=WORK(I+1)
  890 I=I+2
      IF=IF+1
      IFP1=IFP2
      IF(IFP1-NP2)700,900,900
C     COMPLETE A REAL TRANSFORM IN THE 1ST DIMENSION, N EVEN, BY CON-
C     JUGATE SYMMETRIES.
  900 GO TO (1170,1090,1170,910),ICASE
  910 NHALF=N
      N=N+N
      THETA=-TWOPI/FLOAT(N)
      IF(ISIGN)930,920,920
  920 THETA=-THETA
  930 WSTPR=COS(THETA)
      WSTPI=SIN(THETA)
      WR=WSTPR
      WI=WSTPI
      IMIN=3
      JMIN=2*NHALF-1
      GO TO 960
  940 J=JMIN
      DO 950 I=IMIN,NTOT,NP2
      SUMR=(DATA(I)+DATA(J))/2.
      SUMI=(DATA(I+1)+DATA(J+1))/2.
      DIFR=(DATA(I)-DATA(J))/2.
      DIFI=(DATA(I+1)-DATA(J+1))/2.
      TEMPR=WR*SUMI+WI*DIFR
      TEMPI=WI*SUMI-WR*DIFR
      DATA(I)=SUMR+TEMPR
      DATA(I+1)=DIFI+TEMPI
      DATA(J)=SUMR-TEMPR
      DATA(J+1)=-DIFI+TEMPI
  950 J=J+NP2
      IMIN=IMIN+2
      JMIN=JMIN-2
      TEMPR=WR
      WR=WR*WSTPR-WI*WSTPI
      WI=TEMPR*WSTPI+WI*WSTPR
  960 IF(IMIN-JMIN)940,970,1000
  970 IF(ISIGN)980,1000,1000
  980 DO 990 I=IMIN,NTOT,NP2
  990 DATA(I+1)=-DATA(I+1)
 1000 NP2=NP2+NP2
      NTOT=NTOT+NTOT
      J=NTOT+1
      IMAX=NTOT/2+1
 1010 IMIN=IMAX-2*NHALF
      I=IMIN
      GO TO 1030
 1020 DATA(J)=DATA(I)
      DATA(J+1)=-DATA(I+1)
 1030 I=I+2
      J=J-2
      IF(I-IMAX)1020,1040,1040
 1040 DATA(J)=DATA(IMIN)-DATA(IMIN+1)
      DATA(J+1)=0.
      IF(I-J)1060,1080,1080
 1050 DATA(J)=DATA(I)
      DATA(J+1)=DATA(I+1)
 1060 I=I-2
      J=J-2
      IF(I-IMIN)1070,1070,1050
 1070 DATA(J)=DATA(IMIN)+DATA(IMIN+1)
      DATA(J+1)=0.
      IMAX=IMIN
      GO TO 1010
 1080 DATA(1)=DATA(1)+DATA(2)
      DATA(2)=0.
      GO TO 1170
C     COMPLETE A REAL TRANSFORM FOR THE 2ND OR 3RD DIMENSION BY
C     CONJUGATE SYMMETRIES.
 1090 IF(I1RNG-NP1)1100,1170,1170
 1100 DO 1160 I3=1,NTOT,NP2
      I2MAX=I3+NP2-NP1
      DO 1160 I2=I3,I2MAX,NP1
      IMIN=I2+I1RNG
      IMAX=I2+NP1-2
      JMAX=2*I3+NP1-IMIN
      IF(I2-I3)1120,1120,1110
 1110 JMAX=JMAX+NP2
 1120 IF(IDIM-2)1150,1150,1130
 1130 J=JMAX+NP0
      DO 1140 I=IMIN,IMAX,2
      DATA(I)=DATA(J)
      DATA(I+1)=-DATA(J+1)
 1140 J=J-2
 1150 J=JMAX
      DO 1160 I=IMIN,IMAX,NP0
      DATA(I)=DATA(J)
      DATA(I+1)=-DATA(J+1)
 1160 J=J-NP0
C     END OF LOOP ON EACH DIMENSION
 1170 NP0=NP1
      NP1=NP2
 1180 NPREV=N
 1190 RETURN
      END
C ******************************************************************
      SUBROUTINE SDUMMY
C     RANDOM ACCESS I/O ROUTINE TO BE USED WITH SFFTMG SUBROUTINE
C***********************************************************************
      INTEGER OUT
      COMPLEX F(N),G(N)
C     ENTRY TO RECEIVE INPUT SETUP
      ENTRY INSET(IIN)
      IN=IIN
      RETURN
C     ENTRY TO RECEIVE OUTPUT SETUP
      ENTRY OUTSET(IOUT)
      OUT=IOUT
      RETURN
C     ENTRY TO DO INPUT
      ENTRY RANDIN(N,F,NREC)
      READ(IN'NREC)F
      RETURN
C     ENTRY TO DO OUTPUT
      ENTRY RANDOU(N,G,NREC)
      WRITE(OUT'NREC)G
      RETURN
      END
